#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2023 Gabriel Somlo <gsomlo@gmail.com>
# Copyright (c) 2022 Andrew Gillham <gillham@roadsign.com>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.xilinx import Xilinx7SeriesPlatform
from litex.build.openocd import OpenOCD

# This board is available here:
# https://www.aliexpress.com/item/1005001275162791.html

# IOs ----------------------------------------------------------------------------------------------

def _get_io(voltage="2.5V"):
    assert voltage in ["2.5V", "3.3V"]
    VCCIO = {"2.5V": "25", "3.3V": "33"}[voltage]
    _io = [
        # Clk / Rst
        ("cpu_reset_n", 0, Pins("AC16"), IOStandard("LVCMOS15")),

        ("clk50",      0, Pins("F17"), IOStandard("LVCMOS" + VCCIO)),
        ("clk200", 0,
            Subsignal("p", Pins("AB11"), IOStandard("DIFF_SSTL15")),
            Subsignal("n", Pins("AC11"), IOStandard("DIFF_SSTL15"))
        ),
        ("clk156", 0, # TODO verify / test (in docs)
            Subsignal("p", Pins("D6"), IOStandard("LVDS")),
            Subsignal("n", Pins("D5"), IOStandard("LVDS")),
        ),
        ("clk150", 0, # TODO verify / test (in docs)
            Subsignal("p", Pins("F6"), IOStandard("LVDS")),
            Subsignal("n", Pins("F5"), IOStandard("LVDS")),
        ),

        # Leds
        ("user_led_n", 0, Pins("AA2"),  IOStandard("LVCMOS15")),
        ("user_led_n", 1, Pins("AD5"),  IOStandard("LVCMOS15")),
        ("user_led_n", 2, Pins("W10"),  IOStandard("LVCMOS15")),
        ("user_led_n", 3, Pins("Y10"),  IOStandard("LVCMOS15")),
        ("user_led_n", 4, Pins("AE10"), IOStandard("LVCMOS15")),
        ("user_led_n", 5, Pins("W11"),  IOStandard("LVCMOS15")),
        ("user_led_n", 6, Pins("V11"),  IOStandard("LVCMOS15")),
        ("user_led_n", 7, Pins("Y12"),  IOStandard("LVCMOS15")),

        # Buttons
        ("user_btn_n", 0, Pins("AC16"), IOStandard("LVCMOS15")),
        ("user_btn_n", 0, Pins("C24"),  IOStandard("LVCMOS33")), # J4 jumper 2.5V or 3.3V

        # I2C / AT24C04
        ("i2c", 0,
            Subsignal("scl", Pins("U26")),
            Subsignal("sda", Pins("V26")),
            IOStandard("LVCMOS" + VCCIO)
        ),

        # Serial
        ("serial", 0,
            Subsignal("tx",  Pins("L23")),  # CP2102_TX
            Subsignal("rx",  Pins("K21")),  # CP2102_RX
            IOStandard("LVCMOS" + VCCIO)
        ),

        # DDR3 SDRAM
        ("ddram", 0,
            Subsignal("a",     Pins(
                "AB7  AD11 AA8 AF10  AC7 AE11  AC8 AD8",
                "AC13 AF12 AF9 AD10 AE13  AF7 AB12 AC12"),
                IOStandard("SSTL15")),
            Subsignal("ba",    Pins("AE8 AA7 AF13"), IOStandard("SSTL15")),
            Subsignal("ras_n", Pins("Y7"),  IOStandard("SSTL15")),
            Subsignal("cas_n", Pins("AE7"), IOStandard("SSTL15")),
            Subsignal("we_n",  Pins("AF8"),  IOStandard("SSTL15")),
            Subsignal("cs_n",  Pins("AA13"), IOStandard("SSTL15")),
            Subsignal("dm",      Pins(
                "AD16 AB16 AB19 V17 U1 AA3 AD6 AE1"),
                IOStandard("SSTL15")),
            Subsignal("dq",      Pins(
                "AF17 AE17 AF15 AF14 AE15 AD15 AF20 AF19",
                "AA15 AA14 AC14 AD14 AB14 AB15 AA18 AA17",
                "AC18 AD18 AC17 AB17 AA20 AA19 AD19 AC19",
                " W14  V14  V19  V18  V16  W15  W16  Y17",
                "  V4   U6   U5   U2   V3   W3   U7   V6",
                "  Y3   Y2   V2   V1   W1   Y1  AB2  AC2",
                " AA4  AB4  AC4  AC3  AC6  AB6   Y6   Y5",
                " AD4  AD1  AF2  AE2  AE6  AE5  AF3  AE3"),
                IOStandard("SSTL15_T_DCI")),
            Subsignal("dqs_p",   Pins("AE18 Y15 AD20 W18 W6 AB1 AA5 AF5"),
                IOStandard("DIFF_SSTL15_T_DCI")),
            Subsignal("dqs_n",   Pins("AF18 Y16 AE20 W19 W5 AC1 AB5 AF4"),
                IOStandard("DIFF_SSTL15_T_DCI")),
            Subsignal("clk_p",   Pins("AC9"),
                IOStandard("DIFF_SSTL15"), Misc("IO_BUFFER_TYPE=NONE")),
            Subsignal("clk_n",   Pins("AD9"),
                IOStandard("DIFF_SSTL15"), Misc("IO_BUFFER_TYPE=NONE")),
            Subsignal("cke",     Pins("AB10"), IOStandard("SSTL15")),
            Subsignal("odt",     Pins("AA12"), IOStandard("SSTL15")),
            Subsignal("reset_n", Pins("AB20"), IOStandard("LVCMOS15")),
            Misc("SLEW=FAST"),
            Misc("VCCAUX_IO=NORMAL")
        ),
        # 2 Rank Signals:
        # Subsignal("cs_n",  Pins("AD13"), IOStandard("SSTL15")),
        # Subsignal("clk_p", Pins("AA9"), IOStandard("DIFF_SSTL15")),
        # Subsignal("clk_n", Pins("AB9"), IOStandard("DIFF_SSTL15")),
        # Subsignal("cke",   Pins("AA10"), IOStandard("SSTL15")),
        # Subsignal("odt",   Pins("Y13"),  IOStandard("SSTL15")),

        ## TODO verify / test
        # # SPIFlash
        # ("spiflash", 0,
        #     Subsignal("cs_n", Pins("C23")),
        #     Subsignal("clk", Pins("C8")),
        #     Subsignal("dq",   Pins("B24 A25 B22 A22")),
        #     IOStandard("LVCMOS25")
        # ),

        # Sata
        ("sata", 0,
            Subsignal("rx_p", Pins("R4")),
            Subsignal("rx_n", Pins("R3")),
            Subsignal("tx_p", Pins("P2")),
            Subsignal("tx_n", Pins("P1")),
        ),
        ("sata", 1,
            Subsignal("rx_p", Pins("N4")),
            Subsignal("rx_n", Pins("N3")),
            Subsignal("tx_p", Pins("M2")),
            Subsignal("tx_n", Pins("M1")),
        ),

        # SDCard
        ("spisdcard", 0,
            Subsignal("clk",  Pins("V22")),
            Subsignal("cs_n", Pins("W23")),
            Subsignal("mosi", Pins("W24"), Misc("PULLUP True")),
            Subsignal("miso", Pins("U22"), Misc("PULLUP True")),
            Misc("SLEW=FAST"),
            IOStandard("LVCMOS" + VCCIO)
        ),
        ("sdcard", 0,
            Subsignal("clk", Pins("V22")),
            Subsignal("cmd", Pins("W24"), Misc("PULLUP True")),
            Subsignal("data", Pins("U22 V21 W21 W23"), Misc("PULLUP True")),
            Misc("SLEW=FAST"),
            IOStandard("LVCMOS" + VCCIO)
        ),

        # RGMII Ethernet
        ("eth_clocks", 0,
            Subsignal("tx",  Pins("D11"), IOStandard("LVCMOS25")),
            Subsignal("rx",  Pins("C12"), IOStandard("LVCMOS25"))
        ),
        ("eth_clocks", 1,
            Subsignal("tx",  Pins("B11"),  IOStandard("LVCMOS25")),
            Subsignal("rx",  Pins("E10"), IOStandard("LVCMOS25"))
        ),
        ("eth", 0,
            Subsignal("rst_n",   Pins("J8")),
            Subsignal("mdio",    Pins("H11")),
            Subsignal("mdc",     Pins("F9")),
            Subsignal("rx_ctl",  Pins("F8")),
            Subsignal("rx_data", Pins("D8 D9 C9 D10")),
            Subsignal("tx_ctl",  Pins("C14")),
            Subsignal("tx_data", Pins("E12 D13 C13 D14")),
            IOStandard("LVCMOS25")
        ),
        ("eth", 1,
            Subsignal("rst_n",   Pins("B14")),
            Subsignal("mdio",    Pins("B15")),
            Subsignal("mdc",     Pins("A15")),
            Subsignal("rx_ctl",  Pins("A8")),
            Subsignal("rx_data", Pins("B9 A9 B10 A10")),
            Subsignal("tx_ctl",  Pins("A14")),
            Subsignal("tx_data", Pins("B12 A12 A13 C11")),
            IOStandard("LVCMOS25")
        ),

        # HDMI out
        ("hdmi_out", 0,
            Subsignal("clk_p",   Pins("F14"), IOStandard("TMDS_33" if VCCIO == "33" else "LVDS_25")),
            Subsignal("clk_n",   Pins("F13"), IOStandard("TMDS_33" if VCCIO == "33" else "LVDS_25")),
            Subsignal("data0_p", Pins("G12"), IOStandard("TMDS_33" if VCCIO == "33" else "LVDS_25")),
            Subsignal("data0_n", Pins("F12"), IOStandard("TMDS_33" if VCCIO == "33" else "LVDS_25")),
            Subsignal("data1_p", Pins("G10"), IOStandard("TMDS_33" if VCCIO == "33" else "LVDS_25")),
            Subsignal("data1_n", Pins("G9"), IOStandard("TMDS_33" if VCCIO == "33" else "LVDS_25")),
            Subsignal("data2_p", Pins("H9"), IOStandard("TMDS_33" if VCCIO == "33" else "LVDS_25")),
            Subsignal("data2_n", Pins("H8"), IOStandard("TMDS_33" if VCCIO == "33" else "LVDS_25")),
        ),

        # HDMI in
        ("hdmi_in", 0,
            Subsignal("clk_p",   Pins("G11"), IOStandard("TMDS_33" if VCCIO == "33" else "LVDS_25")),
            Subsignal("clk_n",   Pins("F10"), IOStandard("TMDS_33" if VCCIO == "33" else "LVDS_25")),
            Subsignal("data0_p", Pins("J13"), IOStandard("TMDS_33" if VCCIO == "33" else "LVDS_25")),
            Subsignal("data0_n", Pins("H13"), IOStandard("TMDS_33" if VCCIO == "33" else "LVDS_25")),
            Subsignal("data1_p", Pins("J11"), IOStandard("TMDS_33" if VCCIO == "33" else "LVDS_25")),
            Subsignal("data1_n", Pins("J10"), IOStandard("TMDS_33" if VCCIO == "33" else "LVDS_25")),
            Subsignal("data2_p", Pins("H14"), IOStandard("TMDS_33" if VCCIO == "33" else "LVDS_25")),
            Subsignal("data2_n", Pins("G14"), IOStandard("TMDS_33" if VCCIO == "33" else "LVDS_25")),
            Subsignal("scl",     Pins("U21"), IOStandard("LVCMOS" + VCCIO)),
            Subsignal("sda",     Pins("Y20"), IOStandard("LVCMOS" + VCCIO)),
        ),

        # PCIe
        ("pcie_x1", 0,
            Subsignal("rst_n", Pins("E17"), IOStandard("LVCMOS" + VCCIO)),
            Subsignal("clk_p", Pins("H6")),
            Subsignal("clk_n", Pins("H5")),
            Subsignal("rx_p",  Pins("B6")),
            Subsignal("rx_n",  Pins("B5")),
            Subsignal("tx_p",  Pins("A4")),
            Subsignal("tx_n",  Pins("A3"))
        ),
        ("pcie_x2", 0,
            Subsignal("rst_n", Pins("E17"), IOStandard("LVCMOS" + VCCIO)),
            Subsignal("clk_p", Pins("H6")),
            Subsignal("clk_n", Pins("H5")),
            Subsignal("rx_p",  Pins("B6 C4")),
            Subsignal("rx_n",  Pins("B5 C3")),
            Subsignal("tx_p",  Pins("A4 B2")),
            Subsignal("tx_n",  Pins("A3 B1"))
        ),
        ("pcie_x4", 0,
            Subsignal("rst_n", Pins("E17"), IOStandard("LVCMOS" + VCCIO)),
            Subsignal("clk_p", Pins("H6")),
            Subsignal("clk_n", Pins("H5")),
            Subsignal("rx_p",  Pins("B6 C4 E4 G4")),
            Subsignal("rx_n",  Pins("B5 C3 E3 G3")),
            Subsignal("tx_p",  Pins("A4 B2 D2 F2")),
            Subsignal("tx_n",  Pins("A3 B1 D1 F1"))
        ),

        # TODO find / test
        # # SGMII Clk
        # ("sgmii_clock", 0,
        #     Subsignal("p", Pins("")),
        #     Subsignal("n", Pins(""))
        # ),

        # SFP
        ("sfp_a", 0,  # SFP A
            Subsignal("txp", Pins("H2")),
            Subsignal("txn", Pins("H1")),
            Subsignal("rxp", Pins("J4")),
            Subsignal("rxn", Pins("J3")),
            Subsignal("sda", Pins("B21")),
            Subsignal("scl", Pins("C21")),
        ),
        ("sfp_a_tx", 0,  # SFP A
            Subsignal("p", Pins("H2")),
            Subsignal("n", Pins("H1"))
        ),
        ("sfp_a_rx", 0,  # SFP A
            Subsignal("p", Pins("J4")),
            Subsignal("n", Pins("J3"))
        ),
        ("sfp_b", 0,  # SFP B
            Subsignal("txp", Pins("K2")),
            Subsignal("txn", Pins("K1")),
            Subsignal("rxp", Pins("L4")),
            Subsignal("rxn", Pins("L3")),
            Subsignal("sda", Pins("D21")),
            Subsignal("scl", Pins("C22")),
        ),
        ("sfp_b_tx", 0,  # SFP B
            Subsignal("p", Pins("K2")),
            Subsignal("n", Pins("K1"))
        ),
        ("sfp_b_rx", 0,  # SFP B
            Subsignal("p", Pins("L4")),
            Subsignal("n", Pins("L3"))
        ),

        # SI5338 (optional part per seller?)
        ("si5338_i2c", 0,
            Subsignal("sck", Pins("U26"), IOStandard("LVCMOS" + VCCIO)),
            Subsignal("sda", Pins("V26"), IOStandard("LVCMOS" + VCCIO))
        ),
        ("si5338_clkin", 0,  # CLK2A/B
            Subsignal("p", Pins("K6"), IOStandard("LVDS_25")),
            Subsignal("n", Pins("K5"), IOStandard("LVDS_25"))
        ),
    ]

    return _io

# Connectors ---------------------------------------------------------------------------------------

        #GLS FIXME
_connectors = [
    ("LPC", {
        # Row C
        "DP0_C2M_P"     : "", # not connected
        "DP0_C2M_N"     : "", # not connected
        "DP0_M2C_P"     : "", # not connected
        "DP0_M2C_N"     : "", # not connected
        "LA06_P"        : "AE23",
        "LA06_N"        : "AF23",
        "LA10_P"        : "AD26",
        "LA10_N"        : "AE26",
        "LA14_P"        : "Y25",
        "LA14_N"        : "Y26",
        "LA18_CC_P"     : "P23",
        "LA18_CC_N"     : "N23",
        "LA27_P"        : "R26",
        "LA27_N"        : "P26",

        # Row D
        "GBTCLK0_M2C_P" : "", # not connected
        "GBTCLK0_M2C_N" : "", # not connected
        "LA01_CC_P"     : "AA23",
        "LA01_CC_N"     : "AB24",
        "LA05_P"        : "AF24",
        "LA05_N"        : "AF25",
        "LA09_P"        : "AB22",
        "LA09_N"        : "AC22",
        "LA13_P"        : "AB26",
        "LA13_N"        : "AC26",
        "LA17_CC_P"     : "R22",
        "LA17_CC_N"     : "R23",
        "LA23_P"        : "T24",
        "LA23_N"        : "T25",
        "LA26_P"        : "U17",
        "LA26_N"        : "T17",

        # Row G
        "CLK1_M2C_P"    : "Y23",
        "CLK1_M2C_N"    : "AA24",
        "LA00_CC_P"     : "Y22",
        "LA00_CC_N"     : "AA22",
        "LA03_P"        : "U24",
        "LA03_N"        : "U25",
        "LA08_P"        : "AE22",
        "LA08_N"        : "AF22",
        "LA12_P"        : "W25",
        "LA12_N"        : "W26",
        "LA16_P"        : "AA25",
        "LA16_N"        : "AB25",
        "LA20_P"        : "P24",
        "LA20_N"        : "N24",
        "LA22_P"        : "M21",
        "LA22_N"        : "M22",
        "LA25_P"        : "M25",
        "LA25_N"        : "L25",
        "LA29_P"        : "R16",
        "LA29_N"        : "R17",
        "LA31_P"        : "P16",
        "LA31_N"        : "N17",
        "LA33_P"        : "T22",
        "LA33_N"        : "T23",

        # Row H
        "CLK0_M2C_P"    : "AC23",
        "CLK0_M2C_N"    : "AC24",
        "LA02_P"        : "V23",
        "LA02_N"        : "V24",
        "LA04_P"        : "AD21",
        "LA04_N"        : "AE21",
        "LA07_P"        : "AB21",
        "LA07_N"        : "AC21",
        "LA11_P"        : "AD23",
        "LA11_N"        : "AD24",
        "LA15_P"        : "AD25",
        "LA15_N"        : "AE25",
        "LA19_P"        : "R25",
        "LA19_N"        : "P25",
        "LA21_P"        : "U19",
        "LA21_N"        : "U20",
        "LA24_P"        : "T18",
        "LA24_N"        : "T19",
        "LA28_P"        : "R18",
        "LA28_N"        : "P18",
        "LA30_P"        : "M19",
        "LA30_N"        : "V22",
        "LA32_P"        : "T20",
        "LA32_N"        : "R20",
        }
    ),
    ("BTB-A", {
        10: "P19", # P
        11: "P20", # N

        13: "N21", # P
        14: "N22", # N
        15: "K23", # P
        16: "J23", # N

        18: "M24", # P
        19: "L24", # N
        20: "J24", # P
        21: "J25", # N

        23: "H21", # P
        24: "G21", # N
        25: "E21", # P
        26: "E22", # N

        28: "G22", # P
        29: "F23", # N
        30: "E25", # P
        31: "D25", # N

        33: "D23", # P
        34: "D24", # N
        35: "F22", # P
        36: "E23", # N
    }),
    ("BTB-B", {
         5: "W20", # P
         6: "Y21", # N

         8: "N19", # P
         9: "M20", # N
        10: "N26", # P
        11: "M26", # N

        13: "L22", # P
        14: "K22", # N
        15: "K25", # P
        16: "K26", # N

        18: "J21", # P
        19: "H22", # N
        20: "J26", # P
        21: "H26", # N

        23: "G25", # P
        24: "G26", # N
        25: "F25", # P
        26: "E26", # N

        28: "D26", # P
        29: "C26", # N
        30: "A23", # P
        31: "A24", # N

        33: "B20", # P
        34: "A20", # N
        35: "B26",

        38: "K15",
        39: "M16",
    }),
    ("AB", {
         #    N          P
         1: "J20",  2: "K20",
         3: "G20",  4: "H19",
         5: "L20",  6: "L19",
         9: "E20", 10: "F19",
        11: "H18", 12: "H17",
        13: "F18", 14: "G17",
        15: "G16", 16: "H16",
        19: "F24", 20: "G24",
        21: "F20", 22: "G19",
        23: "L18", 24: "M17",
        25: "H24", 26: "H23",
    }),
    ("C", {
         2: "D19", # P
         3: "D20", # N
         7: "D18", # N
         8: "E18", # P
        10: "E16", # N
        11: "E15", # P
    }),
    ("DE", {
         1: "J18",  2: "J19", #  P   N
         3: "L17",  4: "K18", #  P   N
         5: "K16",  6: "K17", #  P   N
         9: "C19", 10: "B19", #  P   N
        11: "C18", 12: "C17", #  N   P
        13: "C16", 14: "B16", #  P   N
        15: "D15", 16: "D16", #  P   N
        19: "G15", 20: "F15", #  P   N
        21: "J15", 22: "J16", #  P   N
        23: "A18", 24: "A19", #  P   N
        25: "B17", 26: "A17", #  P   N
    }),
]
# Platform -----------------------------------------------------------------------------------------

class Platform(Xilinx7SeriesPlatform):
    default_clk_name   = "clk200"
    default_clk_period = 1e9/200e6

    def __init__(self, vccio="3.3V"):
        assert vccio in ["2.5V", "3.3V"]
        Xilinx7SeriesPlatform.__init__(self, "xc7k325t-ffg676-2", _get_io(vccio), _connectors, toolchain="vivado")
        self.add_platform_command("""
set_property CFGBVS VCCO [current_design]
set_property CONFIG_VOLTAGE %s [current_design]
set_property BITSTREAM.GENERAL.COMPRESS TRUE [current_design]
""" % vccio.replace("V", ""))
        self.toolchain.bitstream_commands = ["set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]"]
        self.toolchain.additional_commands = ["write_cfgmem -force -format bin -interface spix4 -size 16 -loadbit \"up 0x0 {build_name}.bit\" -file {build_name}.bin"]

    def create_programmer(self):
        return OpenOCD("openocd_xc7_ft4232.cfg", "bscan_spi_xc7a325t.bit")

    def do_finalize(self, fragment):
        Xilinx7SeriesPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk100",        loose=True), 1e9/100e6)
        self.add_period_constraint(self.lookup_request("clk200",        loose=True), 1e9/200e6)
        self.add_period_constraint(self.lookup_request("eth_clocks:rx", 0, loose=True), 1e9/125e6)
        self.add_period_constraint(self.lookup_request("eth_clocks:tx", 0, loose=True), 1e9/125e6)
        self.add_period_constraint(self.lookup_request("eth_clocks:rx", 1, loose=True), 1e9/125e6)
        self.add_period_constraint(self.lookup_request("eth_clocks:tx", 1, loose=True), 1e9/125e6)
        self.add_platform_command("set_property DCI_CASCADE {{32 34}} [get_iobanks 33]")



#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) Lone Dynamics Corporation <info@lonedynamics.com>
#
# SPDX-License-Identifier: BSD-2-Clause
#

import os
import sys
import json

from migen import *

from litex.gen import *

from litex_boards.platforms import machdyne_schoko

from litex.build.io import DDROutput

from litex.soc.cores.clock import *
from litex.soc.cores.led import LedChaser
from litex.soc.cores.usb_ohci import USBOHCI
from litex.soc.cores.video import VideoVGAPHY
from litex.soc.cores.video import VideoHDMIPHY

from litex.soc.integration.soc_core import *
from litex.soc.integration.builder import *

from litedram.modules import W9825G6KH6
from litedram.phy import GENSDRPHY, HalfRateGENSDRPHY
#from litedram.phy import QuarterRateGENSDRPHY

from litex.soc.integration.soc import SoCRegion

# CRG ---------------------------------------------------------------------------------------------

class _CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq, sdram_rate):
        self.rst        = Signal()
        self.cd_por     = ClockDomain()
        self.cd_sys     = ClockDomain()
        self.cd_video   = ClockDomain()
        self.cd_video5x = ClockDomain()

        # Clk / Rst
        clk48 = platform.request("clk48")

        # Power on reset
        por_count = Signal(16, reset=2**16-1)
        por_done  = Signal()
        self.comb += self.cd_por.clk.eq(clk48)
        self.comb += por_done.eq(por_count == 0)
        self.sync.por += If(~por_done, por_count.eq(por_count - 1))

        # PLL
        self.pll = pll = ECP5PLL()
        self.comb += pll.reset.eq(~por_done | self.rst)
        pll.register_clkin(clk48, 48e6)
        pll.create_clkout(self.cd_sys, sys_clk_freq)

        if sdram_rate == "1:2":
            self.cd_sys2x = ClockDomain()
            self.cd_sys2x_ps = ClockDomain()
            pll.create_clkout(self.cd_sys2x,    2*sys_clk_freq)
            pll.create_clkout(self.cd_sys2x_ps, 2*sys_clk_freq, phase=180)
        elif sdram_rate == "1:4":
            self.cd_sys2x = ClockDomain()
            self.cd_sys4x = ClockDomain()
            self.cd_sys4x_ps = ClockDomain()
            pll.create_clkout(self.cd_sys2x,    2*sys_clk_freq)
            pll.create_clkout(self.cd_sys4x,    4*sys_clk_freq)
            pll.create_clkout(self.cd_sys4x_ps, 4*sys_clk_freq, phase=180)
        else:
            self.cd_sys_ps = ClockDomain()
            pll.create_clkout(self.cd_sys_ps, sys_clk_freq, phase=90)

        if sdram_rate == "1:2":
            sdram_clk = ClockSignal("sys2x_ps")
        elif sdram_rate == "1:4":
            sdram_clk = ClockSignal("sys4x_ps")
        else:
            sdram_clk = ClockSignal("sys_ps")

        self.specials += DDROutput(1, 0, platform.request("sdram_clock"), sdram_clk)
        pll2 = ECP5PLL()
        self.pll2 = pll2
        pll2.register_clkin(clk48, 48e6)
        pll2.create_clkout(self.cd_video, 25e6)
        pll2.create_clkout(self.cd_video5x, 125e6)

        self.cd_usb_12 = ClockDomain()
        self.cd_usb = ClockDomain()
        self.cd_usb_48 = ClockDomain()
        self.cd_usb_48 = self.cd_usb
        pll2.create_clkout(self.cd_usb, 48e6)
        pll2.create_clkout(self.cd_usb_12, 12e6)
        self.comb += pll2.reset.eq(~por_done)

# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    mem_map = {**SoCCore.mem_map, **{
        "usb_ohci":     0xc0000000,
    }}
    def __init__(self, revision="v1", device="45F",  sys_clk_freq=40e6, toolchain="trellis",
        sdram_rate      = "1:2",
        with_led_chaser = True,
        with_usb_host   = False,
        **kwargs):
        platform = machdyne_schoko.Platform(revision=revision, device=device ,toolchain=toolchain)

        # CRG --------------------------------------------------------------------------------------
        self.crg = _CRG(platform, sys_clk_freq, sdram_rate=sdram_rate)

        # SoCCore ----------------------------------------------------------------------------------
        SoCCore.__init__(self, platform, sys_clk_freq, ident="LiteX SoC on Schoko", **kwargs)

        # DRAM -------------------------------------------------------------------------------------
        if not self.integrated_main_ram_size:
            if sdram_rate == "1:2":
                sdrphy_cls = HalfRateGENSDRPHY
            elif sdram_rate == "1:4":
                sdrphy_cls = QuarterRateGENSDRPHY
            else:
                sdrphy_cls = GENSDRPHY

            self.sdrphy = sdrphy_cls(platform.request("sdram"), sys_clk_freq)
            self.add_sdram("sdram",
                phy           = self.sdrphy,
                module        = W9825G6KH6(sys_clk_freq, sdram_rate),
                l2_cache_size = kwargs.get("l2_size", 8192)
            )

        # USB Host ---------------------------------------------------------------------------------
        if with_usb_host:
            self.usb_ohci = USBOHCI(platform, platform.request("usb_host"), usb_clk_freq=int(48e6))
            self.bus.add_slave("usb_ohci_ctrl", self.usb_ohci.wb_ctrl, region=SoCRegion(origin=self.mem_map["usb_ohci"], size=0x100000, cached=False))
            self.dma_bus.add_master("usb_ohci_dma", master=self.usb_ohci.wb_dma)
            self.comb += self.cpu.interrupt[16].eq(self.usb_ohci.interrupt)

        # VGA Framebuffer --------------------------------------------------------------------------
        #self.videophy = VideoVGAPHY(platform.request("vga"),
        #    clock_domain="video")
        #self.add_video_framebuffer(phy=self.videophy, timings="640x480@60Hz",
        #    clock_domain="video", format="rgb565")

        # VGA Terminal -------------------------------------------------------------------------------------
        #self.videophy = VideoVGAPHY(platform.request("vga"),
        #    clock_domain="video")
        #self.add_video_terminal(phy=self.videophy, timings="640x480@60Hz",
        #    clock_domain="video")

        # DDMI Framebuffer -------------------------------------------------------------------------------------
        self.videophy = VideoHDMIPHY(platform.request("ddmi"),
            clock_domain="video")
        self.add_video_framebuffer(phy=self.videophy, timings="640x480@60Hz",
            clock_domain="video", format="rgb565")

        # DDMI Terminal -------------------------------------------------------------------------------------
        #self.videophy = VideoHDMIPHY(platform.request("ddmi"),
        #    clock_domain="video")
        #self.add_video_terminal(phy=self.videophy, timings="640x480@60Hz",
        #    clock_domain="video")

        # Leds -------------------------------------------------------------------------------------
        if with_led_chaser:
            self.leds = LedChaser(
                pads         = platform.request_all("user_led"),
                sys_clk_freq = sys_clk_freq)

# Build --------------------------------------------------------------------------------------------

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=machdyne_schoko.Platform, description="LiteX SoC on Schoko.")
    parser.add_target_argument("--flash",           action="store_true",       help="Flash bitstream to MMOD.")
    parser.add_target_argument("--sys-clk-freq",    default=40e6,  type=float, help="System clock frequency.")
    parser.add_target_argument("--revision",        default="v1",              help="Board Revision (v1, v2).")
    parser.add_target_argument("--device",          default="45F",             help="ECP5 device (25F, 45F or 85F).")
    parser.add_target_argument("--cable",           default="usb-blaster",     help="Specify an openFPGALoader cable.")
    parser.add_target_argument("--with-sdcard",     action="store_true",       help="Enable SDCard support.")
    parser.add_target_argument("--with-spi-sdcard", action="store_true",       help="Enable SPI-mode SDCard support.")
    parser.add_target_argument("--with-usb-host",   action="store_true",       help="Enable USB host support.")
    args = parser.parse_args()

    soc = BaseSoC(
        toolchain    = args.toolchain,
        revision     = args.revision,
        device       = args.device,
        sys_clk_freq = args.sys_clk_freq,
        **parser.soc_argdict)

    if args.with_sdcard:
        soc.add_sdcard()

    if args.with_spi_sdcard:
        soc.add_spi_sdcard()

    builder = Builder(soc, **parser.builder_argdict)

    if args.build:
        builder.build(**parser.toolchain_argdict)

    if args.load:
        prog = soc.platform.create_programmer(args.cable)
        prog.load_bitstream(builder.get_bitstream_filename(mode="sram"))

    if args.flash:
        prog = soc.platform.create_programmer(args.cable)
        prog.flash(0x100000, builder.get_bitstream_filename(mode="sram"))

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2021 Florent Kermarrec <florent@enjoy-digital.fr>
# Copyright (c) 2021 Gwenhael Goavec-Merou <gwenhael.goavec-merou@trabucayre.com>
# SPDX-License-Identifier: BSD-2-Clause

import os

from migen import *

from litex.gen import *

from litex_boards.platforms import quicklogic_quickfeather

from litex.soc.integration.soc import SoCRegion
from litex.soc.integration.soc_core import *
from litex.soc.integration.builder import *
from litex.soc.cores.led import LedChaser
from litex.soc.cores.gpio import *

# CRG ----------------------------------------------------------------------------------------------

class _CRG(LiteXModule):
    def __init__(self, platform, with_eos_s3=False):
        self.rst    = Signal()
        self.cd_sys = ClockDomain()

        # # #

        if with_eos_s3:
            # Use clocks generated by the EOS-S3 CPU.
            self.comb += ClockSignal("sys").eq(ClockSignal("eos_s3_0"))
            self.comb += ResetSignal("sys").eq(ResetSignal("eos_s3_0") | self.rst)
        else:
            # Use clocks generated by the qlal4s3b_cell_macro.
            class Open(Signal): pass
            self.specials += Instance("qlal4s3b_cell_macro",
                o_Sys_Clk0     = self.cd_sys.clk,
                o_Sys_Clk0_Rst = self.cd_sys.rst,
                o_Sys_Clk1     = Open(),
                o_Sys_Clk1_Rst = Open(),
            )

# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    def __init__(self, sys_clk_freq=10e6, with_led_chaser=True, with_gpio_in=True, **kwargs):
        platform = quicklogic_quickfeather.Platform()

        # CRG --------------------------------------------------------------------------------------
        self.crg = _CRG(platform, with_eos_s3=kwargs["cpu_type"] == "eos_s3")

        # SoCCore ----------------------------------------------------------------------------------
        kwargs["with_uart"] = False
        if kwargs.get("cpu_type", None) == "eos_s3":
            kwargs["integrated_sram_size"] = 0
        SoCCore.__init__(self, platform, sys_clk_freq, ident="LiteX SoC on QuickLogic QuickFeather", **kwargs)

        # EOS-S3 Integration -----------------------------------------------------------------------
        if kwargs.get("cpu_type", None) == "eos_s3":
            # in fact SRAM starts at 0x2000_0000 - but for some reason
            # this does not work and most QORC SDK linker scripts
            # use 0x2002_7000 + 0x0003_c800
            self.bus.add_region("sram", SoCRegion(
                origin = 0x2002_7000,
                size   = 0x0003_c800)
            )
            self.bus.add_region("rom", SoCRegion(
                origin = self.mem_map["rom"],
                size   = 4 * 128 * 1024,
                linker = True)
            )

        # Leds -------------------------------------------------------------------------------------
        if with_led_chaser:
            self.leds = LedChaser(
                pads         = platform.request_all("user_led"),
                sys_clk_freq = sys_clk_freq)

        # GPIOIn (Interrupt test) ------------------------------------------------------------------
        if with_gpio_in:
            self.gpio = GPIOIn(platform.request_all("user_btn_n"), with_irq=True)
            if kwargs["cpu_type"] == "eos_s3":
                self.irq.add("gpio", use_loc_if_exists=True)

# Build --------------------------------------------------------------------------------------------

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=quicklogic_quickfeather.Platform, description="LiteX SoC on QuickLogic QuickFeather.")
    parser.set_defaults(cpu_type="eos_s3")
    args = parser.parse_args()

    soc = BaseSoC(**parser.soc_argdict)
    builder = Builder(soc)
    if args.cpu_type == "eos_s3":
        libeos_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "libeos")
        if not os.path.exists(libeos_path):
            os.system("wget -nc https://github.com/litex-hub/litex-boards/files/7880350/libeos.zip")
            os.system(f"unzip libeos.zip -d {libeos_path}")
        builder.add_software_package("libeos", src_dir=libeos_path)
        builder.add_software_library("libeos")
    if args.build:
        builder.build(**parser.toolchain_argdict)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2019 Arnaud Durand <arnaud.durand@unifr.ch>
# Copyright (c) 2022 Martin Hubacek @hubmartin (Twitter)
# Copyright (c) 2022 Raptor Engineering, LLC
# SPDX-License-Identifier: BSD-2-Clause

from migen import *
from migen.genlib.resetsync import AsyncResetSynchronizer

from litex.gen import *

from litex_boards.platforms import rcs_arctic_tern_bmc_card

from litex.soc.cores.clock import *
from litex.soc.integration.soc_core import *
from litex.soc.integration.builder import *
from litex.soc.integration.soc import SoCRegion

from litedram.modules import MT41J256M16
from litedram.phy import ECP5DDRPHY
from liteeth.phy.ecp5rgmii import LiteEthPHYRGMII
from litex.soc.cores.video import VideoGenericPHY

# CRG ----------------------------------------------------------------------------------------------

class _CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq):
        self.rst         = Signal()
        self.cd_init     = ClockDomain()
        self.cd_por      = ClockDomain()
        self.cd_sys      = ClockDomain()
        self.cd_sys2x    = ClockDomain()
        self.cd_sys2x_i  = ClockDomain()
        self.cd_sys2x_eb = ClockDomain()
        self.cd_dvo      = ClockDomain()

        # # #
        self.stop  = Signal()
        self.reset = Signal()

        # Clk / Rst
        clk125 = platform.request("clk125")
        rst_n  = platform.request("rst_n")

        self.clk_inv_alignwd = Signal()
        self.sys_inv_clk_bridge = Signal()
        self.sys_inv_clk_syncb = Signal()

        # Power on reset
        por_count = Signal(16, reset=2**16-1)
        por_done  = Signal()
        self.comb += por_done.eq(por_count == 0)
        self.sync.por += If(~por_done, por_count.eq(por_count - 1))

        # PLL
        sys2x_clk_ecsout = Signal()
        self.pll = pll = ECP5PLL()
        self.comb += pll.reset.eq(~por_done | ~rst_n | self.rst)
        pll.register_clkin(clk125, 125e6)
        pll.create_clkout(self.cd_sys2x_i, 2*sys_clk_freq)
        pll.create_clkout(self.cd_init, 24e6)
        self.specials += [
            Instance("OSCG",
                p_DIV = 128,                # 2.4MHz
                o_OSC = self.cd_por.clk),
            Instance("ECLKBRIDGECS",
                i_CLK0   = self.cd_sys2x_i.clk,
                i_SEL    = 0,
                o_ECSOUT = sys2x_clk_ecsout),
            Instance("ECLKSYNCB",
                i_ECLKI = sys2x_clk_ecsout,
                i_STOP  = self.stop,
                o_ECLKO = self.cd_sys2x.clk),
            Instance("CLKDIVF",
                p_DIV     = "2.0",
                i_ALIGNWD = 0,
                i_CLKI    = self.cd_sys2x.clk,
                i_RST     = self.reset,
                o_CDIVX   = self.cd_sys.clk),
            AsyncResetSynchronizer(self.cd_sys, ~pll.locked | self.reset),
        ]

        # Generate DVO clock
        pll.create_clkout(self.cd_dvo, 40e6)            # 800x600@60
        #pll.create_clkout(self.cd_dvo, 148.35e6)       # 1920x1080@60
        #pll.create_clkout(self.cd_dvo, 148.2e6)        # 1920x1200@60


# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    def __init__(self, sys_clk_freq=50e6, toolchain="trellis",
        with_video_colorbars   = False,
        with_video_terminal    = True,
        with_video_framebuffer = False,
        with_ethernet          = False,
        with_etherbone         = False,
        eth_ip                 = "192.168.1.50",
        **kwargs):
        platform = rcs_arctic_tern_bmc_card.Platform(toolchain=toolchain)

        # CRG --------------------------------------------------------------------------------------
        self.crg = _CRG(platform, sys_clk_freq)

        # SoCCore ----------------------------------------------------------------------------------
        SoCCore.__init__(self, platform, irq_n_irqs=16, clk_freq=sys_clk_freq,
            ident = "LiteX SoC on Arctic Tern (BMC card carrier)",
            **kwargs
        )

        # DDR3 SDRAM -------------------------------------------------------------------------------
        self.ddrphy = ECP5DDRPHY(
            platform.request("ddram"),
            sys_clk_freq=sys_clk_freq)
        self.comb += self.crg.stop.eq(self.ddrphy.init.stop)
        self.comb += self.crg.reset.eq(self.ddrphy.init.reset)
        self.add_sdram("sdram",
            phy           = self.ddrphy,
            module        = MT41J256M16(sys_clk_freq, "1:2"), # Not MT41J256M16, but the AS4C256M16D3C in use has similar specifications
            l2_cache_size = kwargs.get("l2_size", 8192),
        )

        # Ethernet / Etherbone ---------------------------------------------------------------------
        if with_ethernet or with_etherbone:
            self.ethphy = LiteEthPHYRGMII(
                clock_pads = self.platform.request("eth_clocks", 0),
                pads       = self.platform.request("eth", 0),
                tx_delay   = 0e-9,
                rx_delay   = 0e-9)
            if with_ethernet:
                self.add_ethernet(phy=self.ethphy)
            if with_etherbone:
                self.add_etherbone(phy=self.ethphy, ip_address=eth_ip)

        # Video Output -----------------------------------------------------------------------------
        if with_video_colorbars or with_video_terminal or with_video_framebuffer:
            dvo_pads = platform.request("dvo")
            self.videophy = VideoGenericPHY(dvo_pads, clock_domain="dvo", with_clk_ddr_output=False)
            if with_video_terminal:
                #self.add_video_terminal(phy=self.videophy, timings="1920x1080@60Hz", clock_domain="dvo")
                #self.add_video_terminal(phy=self.videophy, timings="1920x1200@60Hz", clock_domain="dvo")
                self.add_video_terminal(phy=self.videophy, timings="800x600@60Hz", clock_domain="dvo")
            elif with_video_framebuffer:
                #self.add_video_framebuffer(phy=self.videophy, timings="1920x1080@60Hz", clock_domain="dvo")
                #self.add_video_framebuffer(phy=self.videophy, timings="1920x1200@60Hz", clock_domain="dvo")
                self.add_video_framebuffer(phy=self.videophy, timings="800x600@60Hz", clock_domain="dvo")
            else:
                self.add_video_colorbars(phy=self.videophy, timings="800x600@60Hz", clock_domain="dvo")

# Build --------------------------------------------------------------------------------------------

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=rcs_arctic_tern_bmc_card.Platform, description="LiteX SoC on Arctic Tern (BMC card carrier).")
    parser.add_target_argument("--sys-clk-freq", default=60e6, type=float, help="System clock frequency (default: 60MHz).")
    ethopts = parser.target_group.add_mutually_exclusive_group()
    ethopts.add_argument("--with-ethernet",  action="store_true",    help="Enable Ethernet support.")
    ethopts.add_argument("--with-etherbone", action="store_true",    help="Enable Etherbone support.")
    parser.add_target_argument("--eth-ip",   default="192.168.1.50", help="Ethernet/Etherbone IP address.")
    args = parser.parse_args()

    soc = BaseSoC(
        toolchain      = args.toolchain,
        sys_clk_freq   = args.sys_clk_freq,
        with_ethernet  = args.with_ethernet,
        with_etherbone = args.with_etherbone,
        eth_ip         = args.eth_ip,
        **parser.soc_argdict)
    builder = Builder(soc, **parser.builder_argdict)
    if args.build:
        builder.build(**parser.toolchain_argdict)

    if args.load:
        prog = soc.platform.create_programmer()
        prog.load_bitstream(builder.get_bitstream_filename(mode="sram", ext=".svf")) # FIXME

if __name__ == "__main__":
    main()



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2023 Hans Baier <hansfbaier@gmail.com>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import Pins, Subsignal, IOStandard, Misc
from litex.build.xilinx import Xilinx7SeriesPlatform
from litex.build.openocd import OpenOCD

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk / Rst
    ("clk50", 0, Pins("U22"), IOStandard("LVCMOS33")),

    ("cpu_reset", 0,     Pins("P4"),   IOStandard("LVCMOS33")),
    ("prog_b", 0,        Pins("AE16"), IOStandard("LVCMOS33")),

    # The core board does not have a USB serial on it,
    # so you will have to attach an USB to serial adapter
    # on these pins
    ("gpio_serial", 0,
        Subsignal("tx", Pins("J2:7")),
        Subsignal("rx", Pins("J2:8")),
        IOStandard("LVCMOS33")
    ),

    # SPIFlash
    # N25Q064A
    ("spiflash4x", 0,  # clock needs to be accessed through STARTUPE2
        Subsignal("cs_n", Pins("P18")),
        Subsignal("clk",  Pins("H13")),
        Subsignal("dq",   Pins("R14 R15 P14 N14")),
        IOStandard("LVCMOS33")
    ),

    # DDR3 SDRAM
    # MT41K128M16JT-125K
    ("ddram", 0,
        Subsignal("a", Pins("E17 G17 F17 C17 G16 D16 H16 E16 H14 F15 F20 H15 C18 G15"),
            IOStandard("SSTL15")),
        Subsignal("ba",    Pins("B17 D18 A17"), IOStandard("SSTL15")),
        Subsignal("ras_n", Pins("A19"), IOStandard("SSTL15")),
        Subsignal("cas_n", Pins("B19"), IOStandard("SSTL15")),
        Subsignal("we_n",  Pins("A18"), IOStandard("SSTL15")),
        # cs_n is hardwired on the board
        #Subsignal("cs_n",  Pins(""), IOStandard("SSTL15")),
        Subsignal("dm", Pins("A22 C22"), IOStandard("SSTL15")),
        Subsignal("dq", Pins(
            "D21 C21 B22 B21 D19 E20 C19 D20 C23 D23 B24 B25 C24 C26 A25 B26"),
            IOStandard("SSTL15"),
            Misc("IN_TERM=UNTUNED_SPLIT_40")),
        Subsignal("dqs_p", Pins("B20 A23"),
            IOStandard("DIFF_SSTL15"),
            Misc("IN_TERM=UNTUNED_SPLIT_40")),
        Subsignal("dqs_n", Pins("A20 A24"),
            IOStandard("DIFF_SSTL15"),
            Misc("IN_TERM=UNTUNED_SPLIT_40")),
        Subsignal("clk_p", Pins("F18"), IOStandard("DIFF_SSTL15")),
        Subsignal("clk_n", Pins("F19"), IOStandard("DIFF_SSTL15")),
        Subsignal("cke",   Pins("E18"), IOStandard("SSTL15")),
        Subsignal("odt",   Pins("G19"), IOStandard("SSTL15")),
        Subsignal("reset_n", Pins("H17"), IOStandard("SSTL15")),
        Misc("SLEW=FAST"),
    ),
]

# The connectors are named after the daughterboard, not the core board
# because on the different core boards the names vary, but on the
# daughterboard they stay the same, which we need to connect the
# daughterboard peripherals to the core board.
# On this board J2 is U2 and J3 is U4
_connectors = [
    ("J2", {
         # odd row     even row
          7: "D26",     8: "E26",
          9: "D25",    10: "E25",
         11: "G26",    12: "H26",
         13: "E23",    14: "F23",
         15: "F22",    16: "G22",
         17: "J26",    18: "J25",
         19: "G21",    20: "G20",
         21: "H22",    22: "H21",
         23: "J21",    24: "K21",
         25: "K26",    26: "K25",
         27: "K23",    28: "K22",
         29: "M26",    30: "N26",
         31: "L23",    32: "L22",
         33: "P26",    34: "R26",
         35: "M25",    36: "M24",
         37: "N22",    38: "N21",
         39: "P24",    40: "P23",
         41: "P25",    42: "R25",
         43: "T25",    44: "T24",
         45: "V21",    46: "U21",
         47: "W23",    48: "V23",
         49: "Y23",    50: "Y22",
         51: "AA25",   52: "Y25",
         53: "AC24",   54: "AB24",
         55: "Y21",    56: "W21",
         57: "Y26",    58: "W25",
         59: "AC26",   60: "AB26",
    }),
    ("J3", {
        # odd row     even row
         7: "B5",   8: "A5",
         9: "B4",  10: "A4",
        11: "A3",  12: "A2",
        13: "D4",  14: "C4",
        15: "C2",  16: "B2",
        17: "E5",  18: "D5",
        19: "C1",  20: "B1",
        21: "E1",  22: "D1",
        23: "F2",  24: "E2",
        25: "G4",  26: "F4",
        27: "G2",  28: "G1",
        29: "J4",  30: "H4",
        31: "H2",  32: "H1",
        33: "H9",  34: "G9",
        35: "M2",  36: "L2",
        37: "L5",  38: "K5",
        39: "M4",  40: "L4",
        41: "N3",  42: "N2",
        43: "M6",  44: "M5",
        45: "K1",  46: "J1",
        47: "R3",  48: "P3",
        49: "T4",  50: "T3",
        51: "P6",  52: "P5",
        53: "N1",  54: "M1",
        55: "R1",  56: "P1",
        57: "T2",  58: "R2",
        59: "U2",  60: "U1",
    })
]

# Platform -----------------------------------------------------------------------------------------

class Platform(Xilinx7SeriesPlatform):
    default_clk_name   = "clk50"
    default_clk_period = 1e9/50e6
    kgates             = None

    def __init__(self, kgates=100, toolchain="vivado", with_daughterboard=False):
        assert(kgates in [75, 100], "kgates can only be 75 or 100 representing a XC7A75T, XC7TA100T")
        self.kgates = kgates
        device = f"xc7a{kgates}tfgg676-1"
        io = _io
        connectors = _connectors

        core_leds_name = "onboard_led" if with_daughterboard else "user_led"
        io += [
            (core_leds_name, 0, Pins("T23"),  IOStandard("LVCMOS33")),
            (core_leds_name, 1, Pins("R23"),  IOStandard("LVCMOS33")),
        ]

        if with_daughterboard:
            from litex_boards.platforms.qmtech_daughterboard import QMTechDaughterboard
            daughterboard = QMTechDaughterboard(IOStandard("LVCMOS33"))
            io += daughterboard.io
            connectors += daughterboard.connectors

        Xilinx7SeriesPlatform.__init__(self, device, io, connectors, toolchain=toolchain)
        self.toolchain.bitstream_commands = \
            ["set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]",
             "set_property BITSTREAM.GENERAL.COMPRESS TRUE [current_design]"]
        self.toolchain.additional_commands = \
            ["write_cfgmem -force -format bin -interface spix4 -size 16 "
             "-loadbit \"up 0x0 {build_name}.bit\" -file {build_name}.bin"]
        self.add_platform_command("set_property INTERNAL_VREF 0.750 [get_iobanks 16]")
        self.add_platform_command("set_property CFGBVS VCCO [current_design]")
        self.add_platform_command("set_property CONFIG_VOLTAGE 3.3 [current_design]")
        self.toolchain.f4pga_device = device

    def create_programmer(self):
        bscan_spi = f"bscan_spi_xc7a{self.kgates}t.bit"
        return OpenOCD("openocd_xc7_ft2232.cfg", bscan_spi)


    def do_finalize(self, fragment):
        Xilinx7SeriesPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk50", loose=True), 1e9/50e6)


#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2021 Hans Baier <hansfbaier@gmail.com>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.altera import AlteraPlatform
from litex.build.altera.programmer import USBBlaster

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk
    ("clk50", 0, Pins("E1"), IOStandard("3.3-V LVTTL")),

    # Button
    ("key", 0, Pins("F3"),  IOStandard("3.3-V LVTTL")),
    ("key", 1, Pins("J6"),  IOStandard("3.3-V LVTTL")),

    # SPIFlash (W25Q64)
    ("spiflash", 0,
        # clk
        Subsignal("cs_n", Pins("D2")),
        Subsignal("clk",  Pins("H1")),
        Subsignal("mosi", Pins("C1")),
        Subsignal("miso", Pins("H2")),
        IOStandard("3.3-V LVTTL"),
    ),

    # SDR SDRAM
    ("sdram_clock", 0, Pins("P2"), IOStandard("3.3-V LVTTL")),
    ("sdram", 0,
        Subsignal("a",     Pins(
            "R7 T7 R8 T8 R6 T5 R5 T4",
            "R4 T3 T6 R3 T2")),
        Subsignal("ba",    Pins("N8 L8")),
        Subsignal("cs_n",  Pins("P8")),
        Subsignal("cke",   Pins("R1")),
        Subsignal("ras_n", Pins("M8")),
        Subsignal("cas_n", Pins("M7")),
        Subsignal("we_n",  Pins("P6")),
        Subsignal("dq", Pins(
            "K5 L3 L4 K6 N3 M6 P3 N5",
            "N2 N1 L1 L2 K1 K2 J1 J2")),
        Subsignal("dm", Pins("N6 P1")),
        IOStandard("3.3-V LVTTL")
    ),
]

# The connectors are named after the daughterboard, not the core board
# because on the different core boards the names vary, but on the
# daughterboard they stay the same, which we need to connect the
# daughterboard peripherals to the core board.
# On this board J2 is U7 and J3 is U8
_connectors = [
    ("J2", {
         # odd row     even row
          7: "G1",    8: "G2",
          9: "D1",   10: "C2",
         11: "B1",   12: "F5",
         13: "D3",   14: "C3",
         15: "B3",   16: "A3",
         17: "B4",   18: "A4",
         19: "E5",   20: "A2",
         21: "D4",   22: "E6",
         23: "C6",   24: "D6",
         25: "B5",   26: "A5",
         27: "B6",   28: "A6",
         29: "B7",   30: "A7",
         31: "D8",   32: "C8",
         33: "D9",   34: "C9",
         35: "B8",   36: "A8",
         37: "B9",   38: "A9",
         39: "E9",   40: "E8",
         41: "E11",  42: "E10",
         43: "A10",  44: "B10",
         45: "D12",  46: "D11",
         47: "B11",  48: "A11",
         49: "B12",  50: "A12",
         51: "B13",  52: "A13",
         53: "B14",  54: "A14",
         55: "D14",  56: "C14",
         57: "B16",  58: "A15",
         59: "C16",  60: "C15",
    }),
    ("J3", {
        # odd row     even row
         7: "R9",     8: "T9",
         9: "R10",   10: "T10",
        11: "R11",   12: "T11",
        13: "R12",   14: "T12",
        15: "N9",    16: "M9",
        17: "M10",   18: "P9",
        19: "P11",   20: "N11",
        21: "R13",   22: "T13",
        23: "T15",   24: "T14",
        25: "N12",   26: "M11",
        27: "R14",   28: "N13",
        29: "N14",   30: "P14",
        31: "P16",   32: "R16",
        33: "N16",   34: "N15",
        35: "M16",   36: "M15",
        37: "L16",   38: "L15",
        39: "P15",   40: "M12",
        41: "L14",   42: "L13",
        43: "K16",   44: "K15",
        45: "K12",   46: "J12",
        47: "J14",   48: "J13",
        49: "K11",   50: "J11",
        51: "G11",   52: "F11",
        53: "F13",   54: "F14",
        55: "F10",   56: "F9",
        57: "E16",   58: "E15",
        59: "D16",   60: "D15",
    })
]

# Platform -----------------------------------------------------------------------------------------

class Platform(AlteraPlatform):
    default_clk_name   = "clk50"
    default_clk_period = 1e9/50e6
    core_resources = [
        ("user_led", 0, Pins("L9"), IOStandard("3.3-V LVTTL")),
        ("serial", 0,
            # Compatible with cheap FT232 based cables (ex: Gaoominy 6Pin Ftdi Ft232Rl Ft232)
            Subsignal("tx", Pins("J3:7"), IOStandard("3.3-V LVTTL")), # GPIO_07 (JP1 Pin 10)
            Subsignal("rx", Pins("J3:8"), IOStandard("3.3-V LVTTL"))  # GPIO_05 (JP1 Pin 8)
        ),
    ]

    def __init__(self, toolchain="quartus", with_daughterboard=False):
        device = "10CL006YU256C8G"
        io = _io
        connectors = _connectors

        if with_daughterboard:
            from litex_boards.platforms.qmtech_daughterboard import QMTechDaughterboard
            daughterboard = QMTechDaughterboard(IOStandard("3.3-V LVTTL"))
            io += daughterboard.io
            connectors += daughterboard.connectors
        else:
            io += self.core_resources

        AlteraPlatform.__init__(self, device, io, connectors, toolchain=toolchain)

        if with_daughterboard:
            # an ethernet pin takes K22, so make it available
            self.add_platform_command("set_global_assignment -name CYCLONEII_RESERVE_NCEO_AFTER_CONFIGURATION \"USE AS REGULAR IO\"")

        # Generate PLL clock in STA
        self.toolchain.additional_sdc_commands.append("derive_pll_clocks")
        # Calculates clock uncertainties
        self.toolchain.additional_sdc_commands.append("derive_clock_uncertainty")

    def create_programmer(self):
        return USBBlaster()

    def do_finalize(self, fragment):
        AlteraPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk50", loose=True), 1e9/50e6)



#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2020 Florent Kermarrec <florent@enjoy-digital.fr>
# SPDX-License-Identifier: BSD-2-Clause

from migen import *

from litex.gen import *

from litex.build.io import CRG

from litex_boards.platforms import tinyfpga_bx

from litex.soc.integration.soc_core import *
from litex.soc.integration.soc import SoCRegion
from litex.soc.integration.builder import *
from litex.soc.cores.led import LedChaser

kB = 1024
mB = 1024*kB

# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    def __init__(self, bios_flash_offset, sys_clk_freq=16e6, with_led_chaser=True, **kwargs):
        platform = tinyfpga_bx.Platform()

        # CRG --------------------------------------------------------------------------------------
        self.crg = CRG(platform.request("clk16"))

        # SoCCore ----------------------------------------------------------------------------------
        # Disable Integrated ROM since too large for iCE40.
        kwargs["integrated_rom_size"]  = 0
        SoCCore.__init__(self, platform, sys_clk_freq, ident="LiteX SoC on TinyFPGA BX", **kwargs)

        # SPI Flash --------------------------------------------------------------------------------
        from litespi.modules import AT25SF081
        from litespi.opcodes import SpiNorFlashOpCodes as Codes
        self.add_spi_flash(mode="1x", module=AT25SF081(Codes.READ_1_1_1), with_master=False)

        # Add ROM linker region --------------------------------------------------------------------
        self.bus.add_region("rom", SoCRegion(
            origin = self.bus.regions["spiflash"].origin + bios_flash_offset,
            size   = 32*kB,
            linker = True)
        )
        self.cpu.set_reset_address(self.bus.regions["rom"].origin)

        # Leds -------------------------------------------------------------------------------------
        if with_led_chaser:
            self.leds = LedChaser(
                pads         = platform.request_all("user_led"),
                sys_clk_freq = sys_clk_freq)

# Build --------------------------------------------------------------------------------------------

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=tinyfpga_bx.Platform, description="LiteX SoC on TinyFPGA BX.")
    parser.add_target_argument("--bios-flash-offset", default="0x50000",         help="BIOS offset in SPI Flash.")
    parser.add_target_argument("--sys-clk-freq",      default=16e6, type=float,  help="System clock frequency.")
    args = parser.parse_args()

    soc = BaseSoC(
         bios_flash_offset = int(args.bios_flash_offset, 0),
         sys_clk_freq      = args.sys_clk_freq,
         **parser.soc_argdict
    )
    builder = Builder(soc, **parser.builder_argdict)
    if args.build:
        builder.build(**parser.toolchain_argdict)

if __name__ == "__main__":
    main()



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2016-2019 Florent Kermarrec <florent@enjoy-digital.fr>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.lattice import LatticePlatform
from litex.build.lattice.programmer import LatticeProgrammer

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk / Rst
    ("clk12", 0, Pins("C8"), IOStandard("LVCMOS33")),
    ("rst_n", 0, Pins("B3"), IOStandard("LVCMOS33")),

    # Leds
    ("user_led", 0, Pins("H11"), IOStandard("LVCMOS33")),
    ("user_led", 1, Pins("J13"), IOStandard("LVCMOS33")),
    ("user_led", 2, Pins("J11"), IOStandard("LVCMOS33")),
    ("user_led", 3, Pins("L12"), IOStandard("LVCMOS33")),
    ("user_led", 4, Pins("K11"), IOStandard("LVCMOS33")),
    ("user_led", 5, Pins("L13"), IOStandard("LVCMOS33")),
    ("user_led", 6, Pins("N15"), IOStandard("LVCMOS33")),
    ("user_led", 7, Pins("P16"), IOStandard("LVCMOS33")),

    # Switches
    ("user_dip_btn", 0, Pins("N2"), IOStandard("LVCMOS33")),
    ("user_dip_btn", 1, Pins("P1"), IOStandard("LVCMOS33")),
    ("user_dip_btn", 2, Pins("M3"), IOStandard("LVCMOS33")),
    ("user_dip_btn", 3, Pins("N1"), IOStandard("LVCMOS33")),

    # Serial
    ("serial", 0,
        Subsignal("tx", Pins("C11"), IOStandard("LVCMOS33")),
        Subsignal("rx", Pins("A11"), IOStandard("LVCMOS33")),
    ),
]

# Platform -----------------------------------------------------------------------------------------

class Platform(LatticePlatform):
    default_clk_name   = "clk12"
    default_clk_period = 1e9/12e6

    def __init__(self, toolchain="diamond"):
        LatticePlatform.__init__(self, "LCMXO3L-6900C-5BG256C", _io, toolchain=toolchain)

    def create_programmer(self):
        _xcf_template = """
<?xml version='1.0' encoding='utf-8' ?>
<!DOCTYPE       ispXCF  SYSTEM  "IspXCF.dtd" >
<ispXCF version="3.6.0">
    <Comment></Comment>
    <Chain>
        <Comm>JTAG</Comm>
        <Device>
            <SelectedProg value="TRUE"/>
            <Pos>1</Pos>
            <Vendor>Lattice</Vendor>
            <Family>MachXO3L</Family>
            <Name>LCMXO3L-6900C</Name>
            <IDCode>0x412bd043</IDCode>
            <Package>All</Package>
            <PON>LCMXO3L-6900C</PON>
            <Bypass>
                <InstrLen>8</InstrLen>
                <InstrVal>11111111</InstrVal>
                <BScanLen>1</BScanLen>
                <BScanVal>0</BScanVal>
            </Bypass>
            <File>{bitstream_file}</File>
            <JedecChecksum>N/A</JedecChecksum>
            <Operation>SRAM Fast Configuration</Operation>
            <Option>
                <SVFVendor>JTAG STANDARD</SVFVendor>
                <IOState>HighZ</IOState>
                <PreloadLength>664</PreloadLength>
                <IOVectorData>0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF</IOVectorData>
                <Usercode>0x00000000</Usercode>
                <AccessMode>SRAM</AccessMode>
            </Option>
        </Device>
    </Chain>
    <ProjectOptions>
        <Program>SEQUENTIAL</Program>
        <Process>ENTIRED CHAIN</Process>
        <OperationOverride>No Override</OperationOverride>
        <StartTAP>TLR</StartTAP>
        <EndTAP>TLR</EndTAP>
        <VerifyUsercode value="FALSE"/>
    </ProjectOptions>
    <CableOptions>
        <CableName>USB2</CableName>
        <PortAdd>FTUSB-0</PortAdd>
        <USBID>Lattice XO3L Starter Kit A Location 0000 Serial A</USBID>
    </CableOptions>
</ispXCF>
"""
        return LatticeProgrammer(_xcf_template)

    def do_finalize(self, fragment):
        LatticePlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk12", loose=True), 1e9/12e6)



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2023 Icenowy Zheng <uwu@icenowy.me>
# SPDX-License-Identifier: BSD-2-Clause

from migen import *

from litex.build.generic_platform import *
from litex.build.gowin.platform import GowinPlatform
from litex.build.gowin.programmer import GowinProgrammer
from litex.build.openfpgaloader import OpenFPGALoader


# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk / Rst.
    ("clk27",  0, Pins("4"), IOStandard("LVCMOS33")),

    # Serial.
    ("serial", 0,
        Subsignal("rx", Pins("70")),
        Subsignal("tx", Pins("69")),
        IOStandard("LVCMOS33")
    ),

    # SPIFlash.
    ("spiflash", 0,
        Subsignal("cs_n", Pins("60"), IOStandard("LVCMOS33")),
        Subsignal("clk",  Pins("59"), IOStandard("LVCMOS33")),
        Subsignal("miso", Pins("62"), IOStandard("LVCMOS33")),
        Subsignal("mosi", Pins("61"), IOStandard("LVCMOS33")),
    ),

    # SDCard.
    ("spisdcard", 0,
        Subsignal("clk",  Pins("83")),
        Subsignal("mosi", Pins("82")),
        Subsignal("cs_n", Pins("81")),
        Subsignal("miso", Pins("84")),
        IOStandard("LVCMOS33"),
    ),
    ("sdcard", 0,
        Subsignal("data", Pins("84 85 80 81")),
        Subsignal("cmd",  Pins("82")),
        Subsignal("clk",  Pins("83")),
        IOStandard("LVCMOS33"),
    ),

    # Leds
    ("led_n", 0,  Pins("15"), IOStandard("LVCMOS33")),
    ("led_n", 1,  Pins("16"), IOStandard("LVCMOS33")),
    ("led_n", 2,  Pins("17"), IOStandard("LVCMOS33")),
    ("led_n", 3,  Pins("18"), IOStandard("LVCMOS33")),
    ("led_n", 4,  Pins("19"), IOStandard("LVCMOS33")),
    ("led_n", 5,  Pins("20"), IOStandard("LVCMOS33")),

    # RGB Led.
    ("rgb_led", 0, Pins("79"), IOStandard("LVCMOS33")),

    # Buttons.
    ("btn", 0,  Pins("88"), IOStandard("LVCMOS33")),
    ("btn", 1,  Pins("87"), IOStandard("LVCMOS33")),

    # SDRAM (embedded in SIP, requires specific IO naming)
    ("O_sdram_clk",   0, Pins(1),  IOStandard("LVCMOS33")),
    ("O_sdram_cke",   0, Pins(1),  IOStandard("LVCMOS33")),
    ("O_sdram_cs_n",  0, Pins(1),  IOStandard("LVCMOS33")),
    ("O_sdram_cas_n", 0, Pins(1),  IOStandard("LVCMOS33")),
    ("O_sdram_ras_n", 0, Pins(1),  IOStandard("LVCMOS33")),
    ("O_sdram_wen_n", 0, Pins(1),  IOStandard("LVCMOS33")),
    ("O_sdram_dqm",   0, Pins(4),  IOStandard("LVCMOS33")),
    ("O_sdram_addr",  0, Pins(11), IOStandard("LVCMOS33")),
    ("O_sdram_ba",    0, Pins(2),  IOStandard("LVCMOS33")),
    ("IO_sdram_dq",   0, Pins(32), IOStandard("LVCMOS33")),
]

# Connectors ---------------------------------------------------------------------------------------

_connectors = [
        # TODO
]

# Platform -----------------------------------------------------------------------------------------

class Platform(GowinPlatform):
    default_clk_name   = "clk27"
    default_clk_period = 1e9/27e6

    def __init__(self, dock="standard", toolchain="gowin"):

        GowinPlatform.__init__(self, "GW2AR-LV18QN88C8/I7", _io, _connectors, toolchain=toolchain, devicename="GW2AR-18C")

        self.toolchain.options["use_mspi_as_gpio"]  = 1
        self.toolchain.options["use_sspi_as_gpio"]  = 1
        self.toolchain.options["use_ready_as_gpio"] = 1
        self.toolchain.options["use_done_as_gpio"]  = 1
        self.toolchain.options["rw_check_on_ram"]   = 1

    def create_programmer(self, kit="openfpgaloader"):
        return OpenFPGALoader(cable="ft2232")

    def do_finalize(self, fragment):
        GowinPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk27", loose=True), 1e9/27e6)



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2021 Miodrag Milanovic <mmicko@gmail.com>
# SPDX-License-Identifier: BSD-2-Clause

# Board schematics at:
# https://github.com/jungle-elec/FireAnt


from litex.build.generic_platform import *
from litex.build.efinix.platform import EfinixPlatform
from litex.build.openfpgaloader import OpenFPGALoader

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk
    ("clk33", 0, Pins("C3"), IOStandard("3.3_V_LVTTL_/_LVCMOS")),

    # Leds
    ("user_led", 0, Pins("C5"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("DRIVE_STRENGTH=1")),
    ("user_led", 1, Pins("B6"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("DRIVE_STRENGTH=1")),
    ("user_led", 2, Pins("C7"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("DRIVE_STRENGTH=1")),
    ("user_led", 3, Pins("A9"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("DRIVE_STRENGTH=1")),

    # Buttons
    ("user_btn", 0, Pins("J9"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("WEAK_PULLUP")),
    ("user_btn", 1, Pins("J8"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("WEAK_PULLUP")),

    # SPIFlash
    ("spiflash", 0,
        Subsignal("cs_n", Pins("J4")),
        Subsignal("clk",  Pins("H4")),
        Subsignal("mosi", Pins("F4")),
        Subsignal("miso", Pins("H3")),
        IOStandard("3.3_V_LVTTL_/_LVCMOS")
    ),
]

# Connectors ---------------------------------------------------------------------------------------

_connectors = [
    ["J1", " - G5 G4 J3 G3 J2 H2 F3 G1 F1 E2 E1 C2 D2 E3 D3 B3 A5 B5 -"],
    ["J2", " -  - H8 G8 H9 G9 F5 F6 F7 E7 F8 E8 D9 B9 C8 B8 A8 C6 A6 -"],
]

# Platform -----------------------------------------------------------------------------------------

class Platform(EfinixPlatform):
    default_clk_name   = "clk33"
    default_clk_period = 1e9/33.33e6

    def __init__(self, toolchain="efinity"):
        EfinixPlatform.__init__(self, "T8F81C2", _io, _connectors, toolchain=toolchain)

    def create_programmer(self):
        return OpenFPGALoader("fireant")

    def do_finalize(self, fragment):
        EfinixPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk33", loose=True), 1e9/33.33e6)



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2019 Arnaud Durand <arnaud.durand@unifr.ch>
# Copyright (c) 2022 Martin Hubacek @hubmartin (Twitter)
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.lattice import LatticeECP5Platform
from litex.build.lattice.programmer import OpenOCDJTAGProgrammer

import os

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk / Rst
    ("clk27",        0, Pins("E17"), IOStandard("LVCMOS33")),
    ("clk100",       0, Pins("C5"),  IOStandard("LVDS")),
    ("ext_clk50",    0, Pins("B11"), IOStandard("LVCMOS33")),
    ("ext_clk50_en", 0, Pins("C11"), IOStandard("LVCMOS33")),
    ("rst_n",        0, Pins("AH1"), IOStandard("LVCMOS33")),

    # Leds
    ("user_led", 0, Pins("AG30"), IOStandard("LVCMOS25")),
    ("user_led", 1, Pins("AK29"), IOStandard("LVCMOS25")),
    ("user_led", 2, Pins("AK30"), IOStandard("LVCMOS25")),
    ("user_led", 3, Pins("AH32"), IOStandard("LVCMOS25")),
    ("user_led", 4, Pins("AG32"), IOStandard("LVCMOS25")),
    ("user_led", 5, Pins("AJ29"), IOStandard("LVCMOS25")),
    ("user_led", 6, Pins("AM28"), IOStandard("LVCMOS25")),
    ("user_led", 7, Pins("AM29"), IOStandard("LVCMOS25")),

    # ws2812
    ("ws2812", 0, Pins("AK31"), IOStandard("LVCMOS25")),

    # Buttons
    ("user_dip_btn", 1, Pins("J1"),  IOStandard("LVCMOS33")),
    ("user_dip_btn", 2, Pins("H1"),  IOStandard("LVCMOS33")),
    ("user_dip_btn", 3, Pins("K1"),  IOStandard("LVCMOS33")),
    ("user_dip_btn", 4, Pins("E15"), IOStandard("LVCMOS25")),
    ("user_dip_btn", 5, Pins("D16"), IOStandard("LVCMOS25")),
    ("user_dip_btn", 6, Pins("B16"), IOStandard("LVCMOS25")),
    ("user_dip_btn", 7, Pins("C16"), IOStandard("LVCMOS25")),
    ("user_dip_btn", 8, Pins("A16"), IOStandard("LVCMOS25")),

    ("button_1", 0, Pins("P4"), IOStandard("LVCMOS25")),

    # Serial
    ("serial", 0,
        Subsignal("rx", Pins("AL28"), IOStandard("LVCMOS25")), ##LVCMOS25
        Subsignal("tx", Pins("AK28"), IOStandard("LVCMOS25")), ##LVCMOS25
    ),

    # DDR3 SDRAM
    ("ddram", 0,
        Subsignal("a", Pins(
            " W4  Y5  AB6  P3  AB5  W5  AC6  Y7",
            "AC7  AD6 W1   Y6  U1   AD7"),
            IOStandard("SSTL135_I")),
        Subsignal("ba",    Pins("U3 Y4 W3"), IOStandard("SSTL135_I")),
        Subsignal("ras_n", Pins("C2"), IOStandard("SSTL135_I")),
        Subsignal("cas_n", Pins("T1"), IOStandard("SSTL135_I")),
        Subsignal("we_n",  Pins("P1"), IOStandard("SSTL135_I")),
        Subsignal("cs_n",  Pins("P2"), IOStandard("SSTL135_I")),
        Subsignal("dm", Pins("AB3 R6 F2 H6"), IOStandard("SSTL135_I")),
        Subsignal("dq", Pins(
            "AC5 AC2 AB4 AE3 W2 AD4 Y1 AB1",
            "V6  P4  V7  T7  U6 T4  U7 U4",
            "H2 K1 F1 L2 E1 K3 H3 H1",
            "N7 J6 L4 K7 P7 L7 K6 H5"),
            IOStandard("SSTL135_I"),
            Misc("TERMINATION=75")),
        Subsignal("dqs_p", Pins("AC3 R4 K2 N3"), IOStandard("SSTL135D_I"),
            Misc("TERMINATION=OFF"),
            Misc("DIFFRESISTOR=100")),
        Subsignal("clk_p", Pins("R3 J4"), IOStandard("SSTL135D_I")),
        Subsignal("cke",   Pins("T2"), IOStandard("SSTL135_I")),
        Subsignal("odt",   Pins("V1"), IOStandard("SSTL135_I")),
        Subsignal("reset_n", Pins("C4"), IOStandard("SSTL135_I")),
        Misc("SLEWRATE=FAST"),
    ),

    # SPIFlash
    ("spiflash", 0,
        Subsignal("cs_n", Pins("AJ3"), IOStandard("LVCMOS33")),
        Subsignal("mosi",   Pins("AK2"), IOStandard("LVCMOS33")),
        Subsignal("miso",   Pins("AJ2"), IOStandard("LVCMOS33")),
        #Subsignal("wp",     Pins("Y2"), IOStandard("LVCMOS33")),
        #Subsignal("hold",   Pins("W1"), IOStandard("LVCMOS33")),
    ),
    ("spiflash4x", 0,
        Subsignal("cs_n", Pins("R2"),          IOStandard("LVCMOS33")),
        Subsignal("dq",   Pins("W2 V2 Y2 W1"), IOStandard("LVCMOS33")),
    ),

    # HDMI
    ("hdmi", 0,
        Subsignal("clk", Pins("E25")),
        Subsignal("hsync_n", Pins("D25")),
        Subsignal("vsync_n", Pins("A25")),
        
        Subsignal("de", Pins("C25")),

        Subsignal("r", Pins("AE27 AD27 AB29 AB30 AB28 AB27 AC26 Y27 D24 W28 F25 F17")),
        Subsignal("g", Pins("AD26 T26 R26 A24 T32 AC30 AB31 V32 W32 Y26 W30 T30")),
        Subsignal("b", Pins("T31 R32 Y32 W31 T29 U28 V27 V26 AC31 AB32 AC32 AD32")),
  
        Subsignal("sda", Pins("AJ1")),
        Subsignal("scl", Pins("AG1")),

        IOStandard("LVCMOS25")
    ),
]

# Platform -----------------------------------------------------------------------------------------

class Platform(LatticeECP5Platform):
    default_clk_name   = "clk27"
    default_clk_period = 1e9/27e6

    def __init__(self, toolchain="trellis", **kwargs):
        LatticeECP5Platform.__init__(self, "LFE5UM-85F-8BG756", _io, toolchain=toolchain, **kwargs)

    def request(self, *args, **kwargs):
        import time
        if "serial" in args:
            msg =  "FT2232H will be used as serial, make sure that:\n"
            msg += " -the hardware has been modified: R22 and R23 should be removed, two 0 Ω resistors shoud be populated on R34 and R35.\n"
            msg += " -the chip is configured as UART with virtual COM on port B (With FTProg or https://github.com/trabucayre/fixFT2232_ecp5evn)."
            print(msg)
            time.sleep(2)
        if "ext_clk50" in args:
            print("An oscillator must be populated on X5.")
            time.sleep(2)

        return LatticeECP5Platform.request(self, *args, **kwargs)

    def create_programmer(self):
        return OpenOCDJTAGProgrammer("openocd_evn_ecp5.cfg")

    def do_finalize(self, fragment):
        LatticeECP5Platform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk27",  loose=True), 1e9/27e6)
        self.add_period_constraint(self.lookup_request("clk100", loose=True), 1e9/100e6)



#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2021 Hans Baier <hansfbaier@gmail.com>
# SPDX-License-Identifier: BSD-2-Clause

# https://www.aliexpress.com/item/4000170003461.html

from migen import *

from litex.gen import *

from litex_boards.platforms import qmtech_artix7_fgg676

from litex.soc.cores.clock import *
from litex.soc.integration.soc import SoCRegion
from litex.soc.integration.soc_core import *
from litex.soc.integration.builder import *
from litex.soc.cores.video import VideoVGAPHY
from litex.soc.cores.led import LedChaser

from litedram.modules import MT41J128M16
from litedram.phy import s7ddrphy

from liteeth.phy.mii import LiteEthPHYMII

# CRG ----------------------------------------------------------------------------------------------

class _CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq, with_ethernet, with_vga):
        self.rst          = Signal()
        self.cd_sys       = ClockDomain()
        self.cd_sys4x     = ClockDomain()
        self.cd_sys4x_dqs = ClockDomain()
        self.cd_idelay    = ClockDomain()
        self.cd_eth       = ClockDomain()
        if with_ethernet:
            self.cd_eth   = ClockDomain()
        if with_vga:
            self.cd_vga   = ClockDomain()

        # # #

        self.pll = pll = S7PLL(speedgrade=-1)
        try:
            reset_button = platform.request("cpu_reset")
            self.comb += pll.reset.eq(~reset_button | self.rst)
        except:
            self.comb += pll.reset.eq(self.rst)

        pll.register_clkin(platform.request("clk50"), 50e6)
        pll.create_clkout(self.cd_sys,       sys_clk_freq)
        pll.create_clkout(self.cd_sys4x,     4*sys_clk_freq)
        pll.create_clkout(self.cd_sys4x_dqs, 4*sys_clk_freq, phase=90)
        pll.create_clkout(self.cd_idelay,    200e6)
        if with_ethernet:
            pll.create_clkout(self.cd_eth,   25e6)
        if with_vga:
            pll.create_clkout(self.cd_vga,   40e6)

        platform.add_false_path_constraints(self.cd_sys.clk, pll.clkin) # Ignore sys_clk to pll.clkin path created by SoC's rst.

        self.idelayctrl = S7IDELAYCTRL(self.cd_idelay)

# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    def __init__(self, toolchain="vivado", kgates=100, sys_clk_freq=100e6, with_daughterboard=False,
        with_ethernet          = False,
        with_etherbone         = False,
        eth_ip                 = "192.168.1.50",
        eth_dynamic_ip         = False,
        with_led_chaser        = True,
        with_video_terminal    = False,
        with_video_framebuffer = False,
        with_jtagbone          = True,
        with_spi_flash         = False,
        **kwargs):
        platform = qmtech_artix7_fgg676.Platform(kgates=kgates, toolchain=toolchain, with_daughterboard=with_daughterboard)

        # CRG --------------------------------------------------------------------------------------
        self.crg = _CRG(platform, sys_clk_freq,
            with_ethernet = (with_ethernet or with_etherbone),
            with_vga      = (with_video_terminal or with_video_framebuffer)
        )

        # SoCCore ----------------------------------------------------------------------------------
        if (kwargs["uart_name"] == "serial") and (not with_daughterboard):
            kwargs["uart_name"] = "gpio_serial"
        SoCCore.__init__(self, platform, sys_clk_freq,
            ident = f"LiteX SoC on QMTech XC7A{kgates}T" + (" + Daughterboard" if with_daughterboard else ""),
            **kwargs)

        # DDR3 SDRAM -------------------------------------------------------------------------------
        if not self.integrated_main_ram_size:
            self.ddrphy = s7ddrphy.A7DDRPHY(platform.request("ddram"),
                memtype        = "DDR3",
                nphases        = 4,
                sys_clk_freq   = sys_clk_freq)
            self.add_sdram("sdram",
                phy           = self.ddrphy,
                module        = MT41J128M16(sys_clk_freq, "1:4"),
                l2_cache_size = kwargs.get("l2_size", 8192)
            )

        # Ethernet / Etherbone ---------------------------------------------------------------------
        if with_ethernet or with_etherbone:
            self.ethphy = LiteEthPHYMII(
                clock_pads = self.platform.request("eth_clocks"),
                pads       = self.platform.request("eth"))
            if with_ethernet:
                self.add_ethernet(phy=self.ethphy, dynamic_ip=eth_dynamic_ip)
            if with_etherbone:
                self.add_etherbone(phy=self.ethphy, ip_address=eth_ip)
            # The daughterboard has the tx clock wired to a non-clock pin, so we can't help it
            self.platform.add_platform_command("set_property CLOCK_DEDICATED_ROUTE FALSE [get_nets eth_clocks_tx_IBUF]")

        # Jtagbone ---------------------------------------------------------------------------------
        if with_jtagbone:
            self.add_jtagbone()

        # SPI Flash --------------------------------------------------------------------------------
        if with_spi_flash:
            from litespi.modules import MT25QL128
            from litespi.opcodes import SpiNorFlashOpCodes as Codes
            self.add_spi_flash(mode="4x", module=MT25QL128(Codes.READ_1_1_1), with_master=True)

        # Video ------------------------------------------------------------------------------------
        if with_video_terminal or with_video_framebuffer:
            self.videophy = VideoVGAPHY(platform.request("vga"), clock_domain="vga")
            if with_video_terminal:
                self.add_video_terminal(phy=self.videophy, timings="800x600@60Hz", clock_domain="vga")
            if with_video_framebuffer:
                self.add_video_framebuffer(phy=self.videophy, timings="800x600@60Hz", clock_domain="vga")

        # Leds -------------------------------------------------------------------------------------
        if with_led_chaser:
            self.leds = LedChaser(
                pads         = platform.request_all("user_led"),
                sys_clk_freq = sys_clk_freq)

        if not with_daughterboard and kwargs["uart_name"] == "serial":
            kwargs["uart_name"] = "jtag_serial"

# Build --------------------------------------------------------------------------------------------

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=qmtech_artix7_fgg676.Platform, description="LiteX SoC on QMTech XC7AXXXT.")
    parser.add_target_argument("--kgates",              default=100,   type=int,  help="Number of kgates. Allowed values: 75, 100, 200, representing XC7A75T, XC7A100T and XC7A200T")
    parser.add_target_argument("--sys-clk-freq",        default=100e6, type=float,  help="System clock frequency.")
    parser.add_target_argument("--with-daughterboard",  action="store_true",        help="Board plugged into the QMTech daughterboard.")
    ethopts = parser.target_group.add_mutually_exclusive_group()
    ethopts.add_argument("--with-ethernet",        action="store_true",    help="Enable Ethernet support.")
    ethopts.add_argument("--with-etherbone",       action="store_true",    help="Enable Etherbone support.")
    parser.add_target_argument("--eth-ip",         default="192.168.1.50", help="Ethernet/Etherbone IP address.")
    parser.add_target_argument("--eth-dynamic-ip", action="store_true",    help="Enable dynamic Ethernet IP addresses setting.")
    sdopts = parser.target_group.add_mutually_exclusive_group()
    sdopts.add_argument("--with-spi-sdcard",       action="store_true", help="Enable SPI-mode SDCard support.")
    sdopts.add_argument("--with-sdcard",           action="store_true", help="Enable SDCard support.")
    parser.add_target_argument("--with-jtagbone",  action="store_true", help="Enable Jtagbone support.")
    parser.add_target_argument("--with-spi-flash", action="store_true", help="Enable SPI Flash (MMAPed).")
    viopts = parser.target_group.add_mutually_exclusive_group()
    viopts.add_argument("--with-video-terminal",    action="store_true", help="Enable Video Terminal (VGA).")
    viopts.add_argument("--with-video-framebuffer", action="store_true", help="Enable Video Framebuffer (VGA).")
    args = parser.parse_args()

    soc = BaseSoC(
        toolchain              = args.toolchain,
        kgates                 = args.kgates,
        sys_clk_freq           = args.sys_clk_freq,
        with_daughterboard     = args.with_daughterboard,
        with_ethernet          = args.with_ethernet,
        with_etherbone         = args.with_etherbone,
        eth_ip                 = args.eth_ip,
        eth_dynamic_ip         = args.eth_dynamic_ip,
        with_jtagbone          = args.with_jtagbone,
        with_spi_flash         = args.with_spi_flash,
        with_video_terminal    = args.with_video_terminal,
        with_video_framebuffer = args.with_video_framebuffer,
        **parser.soc_argdict
    )

    if args.with_spi_sdcard:
        soc.add_spi_sdcard()
    if args.with_sdcard:
        soc.add_sdcard()

    builder = Builder(soc, **parser.builder_argdict)
    if args.build:
        builder.build(**parser.toolchain_argdict)

    if args.load:
        prog = soc.platform.create_programmer()
        prog.load_bitstream(builder.get_bitstream_filename(mode="sram"))

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2020-2022 Florent Kermarrec <florent@enjoy-digital.fr>
# SPDX-License-Identifier: BSD-2-Clause

# Build/Use:
# python3 -m litex_boards.targets.sqrl_fk33 --with-hbm --sys-clk-freq=250e6 --csr-csv=csr.csv --build --load
# litex_server --jtag --jtag-config=openocd_xc7_ft2232.cfg --jtag-chain=2
# litex_term crossover

import os

from migen import *

from litex.gen import *

from litex_boards.platforms import sqrl_fk33

from litex.soc.cores.clock import *
from litex.soc.integration.soc_core import *
from litex.soc.integration.builder import *
from litex.soc.integration.soc import SoCRegion
from litex.soc.interconnect.axi import *
from litex.soc.cores.ram.xilinx_usp_hbm2 import USPHBM2
from litex.soc.cores.led import LedChaser

from litepcie.phy.usppciephy import USPHBMPCIEPHY
from litepcie.core import LitePCIeEndpoint, LitePCIeMSI
from litepcie.frontend.dma import LitePCIeDMA
from litepcie.frontend.wishbone import LitePCIeWishboneBridge
from litepcie.software import generate_litepcie_software

# CRG ----------------------------------------------------------------------------------------------

class _CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq, with_hbm):
        self.rst    = Signal()
        self.cd_sys = ClockDomain()
        if with_hbm:
            self.cd_hbm_ref = ClockDomain()
            self.cd_apb     = ClockDomain()

        # # #

        self.pll = pll = USPMMCM(speedgrade=-2)
        self.comb += pll.reset.eq(self.rst)
        pll.register_clkin(platform.request("clk200"), 200e6)
        pll.create_clkout(self.cd_sys, sys_clk_freq)
        platform.add_false_path_constraints(self.cd_sys.clk, pll.clkin) # Ignore sys_clk to pll.clkin path created by SoC's rst.

        if with_hbm:
            pll.create_clkout(self.cd_hbm_ref, 100e6)
            pll.create_clkout(self.cd_apb,     100e6)

# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    def __init__(self, sys_clk_freq=125e6,
        with_led_chaser = True,
        with_pcie       = False,
        with_hbm        = False,
        **kwargs):
        platform = sqrl_fk33.Platform()
        if with_hbm:
            assert 225e6 <= sys_clk_freq <= 450e6

        # CRG --------------------------------------------------------------------------------------
        self.crg = _CRG(platform, sys_clk_freq, with_hbm)

        # SoCCore ----------------------------------------------------------------------------------
        if kwargs.get("uart_name", "serial") == "serial":
            kwargs["uart_name"] = "crossover" # Defaults to Crossover-UART.
        SoCCore.__init__(self, platform, sys_clk_freq, ident="LiteX SoC on FK33", **kwargs)

        # JTAGBone --------------------------------------------------------------------------------
        self.add_jtagbone(chain=2) # Chain 1 already used by HBM2 debug probes.

        # HBM --------------------------------------------------------------------------------------
        if with_hbm:
            # Add HBM Core.
            self.hbm = hbm = ClockDomainsRenamer({"axi": "sys"})(USPHBM2(platform))

            # Get HBM .xci.
            os.system("wget https://github.com/litex-hub/litex-boards/files/8178874/hbm_0.xci.txt")
            os.makedirs("ip/hbm", exist_ok=True)
            os.system("mv hbm_0.xci.txt ip/hbm/hbm_0.xci")

            # Connect four of the HBM's AXI interfaces to the main bus of the SoC.
            for i in range(4):
                axi_hbm      = hbm.axi[i]
                axi_lite_hbm = AXILiteInterface(data_width=256, address_width=33)
                self.submodules += AXILite2AXI(axi_lite_hbm, axi_hbm)
                self.bus.add_slave(f"hbm{i}", axi_lite_hbm, SoCRegion(origin=0x4000_0000 + 0x1000_0000*i, size=0x1000_0000)) # 256MB.
            # Link HBM2 channel 0 as main RAM
            self.bus.add_region("main_ram", SoCRegion(origin=0x4000_0000, size=0x1000_0000, linker=True)) # 256MB.

        # PCIe -------------------------------------------------------------------------------------
        if with_pcie:
            assert self.csr_data_width == 32
            # PHY
            self.pcie_phy = USPHBMPCIEPHY(platform, platform.request("pcie_x4"),
                data_width = 128,
                bar0_size  = 0x20000)

            # Endpoint
            self.pcie_endpoint = LitePCIeEndpoint(self.pcie_phy, max_pending_requests=8)

            # Wishbone bridge
            self.pcie_bridge = LitePCIeWishboneBridge(self.pcie_endpoint,
                base_address = self.mem_map["csr"])
            self.bus.add_master(master=self.pcie_bridge.wishbone)

            # DMA0
            self.pcie_dma0 = LitePCIeDMA(self.pcie_phy, self.pcie_endpoint,
                with_buffering = True, buffering_depth=1024,
                with_loopback  = True)

            self.add_constant("DMA_CHANNELS", 1)

            # MSI
            self.pcie_msi = LitePCIeMSI()
            self.comb += self.pcie_msi.source.connect(self.pcie_phy.msi)
            self.interrupts = {
                "PCIE_DMA0_WRITER":    self.pcie_dma0.writer.irq,
                "PCIE_DMA0_READER":    self.pcie_dma0.reader.irq,
            }
            for i, (k, v) in enumerate(sorted(self.interrupts.items())):
                self.comb += self.pcie_msi.irqs[i].eq(v)
                self.add_constant(k + "_INTERRUPT", i)

        # Leds -------------------------------------------------------------------------------------
        if with_led_chaser:
            self.leds = LedChaser(
                pads         = platform.request_all("user_led"),
                sys_clk_freq = sys_clk_freq)

# Build --------------------------------------------------------------------------------------------

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=sqrl_fk33.Platform, description="LiteX SoC on FK33.")
    parser.add_target_argument("--sys-clk-freq", default=125e6, type=float, help="System clock frequency.")
    parser.add_target_argument("--with-pcie",    action="store_true",       help="Enable PCIe support.")
    parser.add_target_argument("--with-hbm",     action="store_true",       help="Use HBM2.")
    parser.add_target_argument("--driver",       action="store_true",       help="Generate PCIe driver.")
    args = parser.parse_args()

    soc = BaseSoC(
        sys_clk_freq = args.sys_clk_freq,
        with_pcie    = args.with_pcie,
        with_hbm     = args.with_hbm,
        **parser.soc_argdict
    )
    builder = Builder(soc, **parser.builder_argdict)
    if args.build:
        builder.build(**parser.toolchain_argdict)

    if args.driver:
        generate_litepcie_software(soc, os.path.join(builder.output_dir, "driver"))

    if args.load:
        prog = soc.platform.create_programmer()
        prog.load_bitstream(builder.get_bitstream_filename(mode="sram"))

if __name__ == "__main__":
    main()



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2020 Basel Sayeh <Basel.Sayeh@hotmail.com>
# Copyright (c) 2021 Hans Baier <hansfbaier@gmail.com>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.altera import AlteraPlatform
from litex.build.altera.programmer import USBBlaster

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk
    ("clk50", 0, Pins("B14"), IOStandard("3.3-V LVTTL")),

    # Button
    ("key", 0, Pins("AD23"),  IOStandard("3.3-V LVTTL")),
    ("key", 1, Pins("AD24"),  IOStandard("3.3-V LVTTL")),

    # SPIFlash (W25Q64)
    ("spiflash", 0,
        # clk
        Subsignal("cs_n", Pins("D5")),
        Subsignal("clk",  Pins("F6")),
        Subsignal("mosi", Pins("E6")),
        Subsignal("miso", Pins("D6")),
        IOStandard("3.3-V LVTTL"),
    ),

    # SDR SDRAM
    ("sdram_clock", 0, Pins("J23"), IOStandard("3.3-V LVTTL")),
    ("sdram", 0,
        Subsignal("a",     Pins(
            "L25 L26 M25 M26 N22 N23 N24 M22",
            "M24 L23 K26 L24 K23")),
        Subsignal("ba",    Pins("J25 J26")),
        Subsignal("cs_n",  Pins("H26")),
        Subsignal("cke",   Pins("K24")),
        Subsignal("ras_n", Pins("H25")),
        Subsignal("cas_n", Pins("G26")),
        Subsignal("we_n",  Pins("G25")),
        Subsignal("dq", Pins(
            "B25 B26 C25 C26 D25 D26 E25 E26",
            "H23 G24 G22 F24 F23 E24 D24 C24")),
        Subsignal("dm", Pins("F26 H24")),
        IOStandard("3.3-V LVTTL")
    ),
]

# The connectors are named after the daughterboard, not the core board
# because on the different core boards the names vary, but on the
# daughterboard they stay the same, which we need to connect the
# daughterboard peripherals to the core board.
# On this board J2 is U5 and J3 is U4
_connectors = [
    ("J2", {
        # odd row      even row
         7: "AF24",   8: "AF25",
         9: "AC21",  10: "AD21",
        11: "AE23",  12: "AF23",
        13: "AE22",  14: "AF22",
        15: "AD20",  16: "AE21",
        17: "AF20",  18: "AF21",
        19: "AE19",  20: "AF19",
        21: "AC19",  22: "AD19",
        23: "AE18",  24: "AF18",
        25: "AC18",  26: "AD18",
        27: "AE17",  28: "AF17",
        29: "AC17",  30: "AD17",
        31: "AF15",  32: "AF16",
        33: "AC16",  34: "AD16",
        35: "AE14",  36: "AE15",
        37: "AC15",  38: "AD15",
        39: "AC14",  40: "AD14",
        41: "AF11",  42: "AF12",
        43: "AC10",  44: "AD10",
        45: "AE9",   46: "AF9",
        47: "AF7",   48: "AF8",
        49: "AE7",   50: "AF6",
        51: "AE5",   52: "AE6",
        53: "AD5",   54: "AD6",
        55: "AF4",   56: "AF5",
        57: "AD3",   58: "AE3",
        59: "AC4",   60: "AD4",
    }),
    ("J3", {
        # odd row      even row
         7: "C21",    8: "B22",
         9: "B23",   10: "A23",
        11: "B21",   12: "A22",
        13: "C19",   14: "B19",
        15: "A21",   16: "A20",
        17: "A19",   18: "A18",
        19: "C17",   20: "B18",
        21: "C16",   22: "B17",
        23: "A17",   24: "A16",
        25: "B15",   26: "A15",
        27: "C15",   28: "C14",
        29: "C13",   30: "B13",
        31: "C12",   32: "C11",
        33: "A13",   34: "A12",
        35: "B11",   36: "A11",
        37: "B10",   38: "A10",
        39: "C10",   40: "B9",
        41: "A9",    42: "A8",
        43: "A7",    44: "A6",
        45: "B7",    46: "B6",
        47: "B5",    48: "A5",
        49: "B4",    50: "A4",
        51: "C5",    52: "C4",
        53: "A3",    54: "A2",
        55: "B2",    56: "B1",
        57: "D1",    58: "C1",
        59: "E2",    60: "E1",
    })
]

# Platform -----------------------------------------------------------------------------------------

class Platform(AlteraPlatform):
    default_clk_name   = "clk50"
    default_clk_period = 1e9/50e6
    core_resources = [
        ("user_led", 0, Pins("A25"), IOStandard("3.3-V LVTTL")),
        ("user_led", 1, Pins("A24"), IOStandard("3.3-V LVTTL")),
        ("serial", 0,
            Subsignal("tx", Pins("J3:7"), IOStandard("3.3-V LVTTL")),
            Subsignal("rx", Pins("J3:8"), IOStandard("3.3-V LVTTL"))
        ),
    ]

    def __init__(self, toolchain="quartus", with_daughterboard=False):
        device = "EP4CGX150DF27I7"
        io = _io
        connectors = _connectors

        if with_daughterboard:
            from litex_boards.platforms.qmtech_daughterboard import QMTechDaughterboard
            daughterboard = QMTechDaughterboard(IOStandard("3.3-V LVTTL"))
            io += daughterboard.io
            connectors += daughterboard.connectors
        else:
            io += self.core_resources

        AlteraPlatform.__init__(self, device, io, connectors, toolchain=toolchain)

        if with_daughterboard:
            # an ethernet pin takes K22, so make it available
            self.add_platform_command("set_global_assignment -name CYCLONEII_RESERVE_NCEO_AFTER_CONFIGURATION \"USE AS REGULAR IO\"")

    def create_programmer(self):
        return USBBlaster()

    def do_finalize(self, fragment):
        AlteraPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk50", loose=True), 1e9/50e6)



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2020 Antmicro <www.antmicro.com>
# Copyright (c) 2019 David Shah <dave@ds0.me>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.xilinx import XilinxUSPPlatform, VivadoProgrammer

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk / Rst
    ("clk125", 0,
        Subsignal("p", Pins("F23"), IOStandard("LVDS")),
        Subsignal("n", Pins("E23"), IOStandard("LVDS")),
    ),
    ("clk300", 0,
        Subsignal("p", Pins("AH18"), IOStandard("DIFF_SSTL12_DCI")),
        Subsignal("n", Pins("AH17"), IOStandard("DIFF_SSTL12_DCI")),
    ),
    ("cpu_reset", 0, Pins("M11"), IOStandard("LVCMOS33")),

    # Leds
    ("user_led", 0, Pins("D5"), IOStandard("LVCMOS33")),
    ("user_led", 1, Pins("D6"), IOStandard("LVCMOS33")),
    ("user_led", 2, Pins("A5"), IOStandard("LVCMOS33")),
    ("user_led", 3, Pins("B5"), IOStandard("LVCMOS33")),

    # Buttons
    ("user_btn", 0, Pins("B4"), IOStandard("LVCMOS33")),
    ("user_btn", 1, Pins("C4"), IOStandard("LVCMOS33")),
    ("user_btn", 2, Pins("B3"), IOStandard("LVCMOS33")),
    ("user_btn", 3, Pins("C3"), IOStandard("LVCMOS33")),

    # Switches
    ("user_dip", 0, Pins("E4"), IOStandard("LVCMOS33")),
    ("user_dip", 1, Pins("D4"), IOStandard("LVCMOS33")),
    ("user_dip", 2, Pins("F5"), IOStandard("LVCMOS33")),
    ("user_dip", 3, Pins("F4"), IOStandard("LVCMOS33")),

    # Serial
    ("serial", 0,
        Subsignal("cts", Pins("A19")),
        Subsignal("rts", Pins("C18")),
        Subsignal("tx", Pins("C19")),
        Subsignal("rx", Pins("A20")),
        IOStandard("LVCMOS18")
    ),

    # I2C
    ("i2c", 0,
        Subsignal("sda", Pins("P12")),
        Subsignal("scl", Pins("N12")),
        IOStandard("LVCMOS33")
    ),

    # DDR4 SDRAM
    ("ddram", 0,
        Subsignal("a",       Pins(
            "AH16 AG14 AG15 AF15 AF16 AJ14 AH14 AF17",
            "AK17 AJ17 AK14 AK15 AL18 AK18"),
            IOStandard("SSTL12_DCI")),
        Subsignal("ba",      Pins("AL15 AL16"), IOStandard("SSTL12_DCI")),
        Subsignal("bg",      Pins("AC16 AB16"), IOStandard("SSTL12_DCI")),
        Subsignal("ras_n",   Pins("AD15"), IOStandard("SSTL12_DCI")), # A16
        Subsignal("cas_n",   Pins("AA14"), IOStandard("SSTL12_DCI")), # A15
        Subsignal("we_n",    Pins("AA16"), IOStandard("SSTL12_DCI")), # A14
        Subsignal("cs_n",    Pins("AA15"), IOStandard("SSTL12_DCI")), # also AL17 AN17 AN16 for larger SODIMMs
        Subsignal("act_n",   Pins("AC17"), IOStandard("SSTL12_DCI")),
        #Subsignal("alert_n", Pins("AB15"), IOStandard("SSTL12_DCI")),
        #Subsignal("par",     Pins("AD16"), IOStandard("SSTL12_DCI")),
        Subsignal("dm",      Pins("AH22 AE18 AL20 AP19 AF11 AH12 AK13 AN12"),
            IOStandard("POD12_DCI")),
        Subsignal("dq",      Pins(
                "AE24 AE23 AF22 AF21 AG20 AG19 AH21 AG21",
                "AA20 AA19 AD19 AC18 AE20 AD20 AC19 AB19",
                "AJ22 AJ21 AK20 AJ20 AK19 AJ19 AL23 AL22",
                "AN23 AM23 AP23 AN22 AP22 AP21 AN19 AM19",
                "AC13 AB13 AF12 AE12 AF13 AE13 AE14 AD14",
                "AG8  AF8  AG10 AG11 AH13 AG13 AJ11 AH11",
                "AK9  AJ9  AK10 AJ10 AL12 AK12 AL10 AL11",
                "AM8  AM9  AM10 AM11 AP11 AN11 AP9  AP10"),
            IOStandard("POD12_DCI"),
            Misc("PRE_EMPHASIS=RDRV_240"),
            Misc("EQUALIZATION=EQ_LEVEL2")),
        Subsignal("dqs_p",   Pins("AF23 AA18 AK22 AM21 AC12 AG9 AK8 AN9"),
            IOStandard("DIFF_POD12_DCI"),
            Misc("PRE_EMPHASIS=RDRV_240"),
            Misc("EQUALIZATION=EQ_LEVEL2")),
        Subsignal("dqs_n",   Pins("AG23 AB18 AK23 AN21 AD12 AH9 AL8 AN8"),
            IOStandard("DIFF_POD12_DCI"),
            Misc("PRE_EMPHASIS=RDRV_240"),
            Misc("EQUALIZATION=EQ_LEVEL2")),
        Subsignal("clk_p",   Pins("AF18"), IOStandard("DIFF_SSTL12_DCI")), # also AJ16 for larger SODIMMs
        Subsignal("clk_n",   Pins("AG18"), IOStandard("DIFF_SSTL12_DCI")), # also AJ15 for larger SODIMMs
        Subsignal("cke",     Pins("AD17"), IOStandard("SSTL12_DCI")), # also AM15 for larger SODIMMs
        Subsignal("odt",     Pins("AE15"), IOStandard("SSTL12_DCI")), # also AM16 for larger SODIMMs
        Subsignal("reset_n", Pins("AB14"), IOStandard("LVCMOS12")),
        Misc("SLEW=FAST"),
    ),
]

# Platform -----------------------------------------------------------------------------------------

class Platform(XilinxUSPPlatform):
    default_clk_name   = "clk125"
    default_clk_period = 1e9/125e6

    def __init__(self, toolchain="vivado"):
        XilinxUSPPlatform.__init__(self, "xczu7ev-ffvc1156-2-i", _io, toolchain=toolchain)

    def create_programmer(self):
        return VivadoProgrammer()

    def do_finalize(self, fragment):
        XilinxUSPPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk125", loose=True), 1e9/125e6)
        self.add_period_constraint(self.lookup_request("clk300", loose=True), 1e9/125e6)
        self.add_platform_command("set_property INTERNAL_VREF 0.84 [get_iobanks 64]")
        self.add_platform_command("set_property INTERNAL_VREF 0.84 [get_iobanks 65]")
        self.add_platform_command("set_property INTERNAL_VREF 0.84 [get_iobanks 66]")



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2023 Kazumoto Kojima
# Copyright (c) 2023 Hans Baier <hansfbaier@gmail.com>
#
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import Pins, Subsignal, IOStandard, Misc
from litex.build.xilinx import XilinxPlatform
from litex.build.openocd import OpenOCD

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk / Rst
    ("clk50", 0, Pins("F22"), IOStandard("LVCMOS33")),

    # The core board does not have a USB serial on it,
    # so you will have to attach an USB to serial adapter
    # on these pins
    ("gpio_serial", 0,
        Subsignal("tx", Pins("J2:7")),
        Subsignal("rx", Pins("J2:8")),
        IOStandard("LVCMOS33")
    ),

    # SPIFlash
    # S25FL256L
    ("spiflash4x", 0,  # clock needs to be accessed through STARTUPE2
        Subsignal("cs_n", Pins("C23")),
        Subsignal("clk",  Pins("C8")),
        Subsignal("dq",   Pins("B24", "A25", "B22", "A22")),
        IOStandard("LVCMOS33")
    ),

    # DDR3 SDRAM
    # MT41J128M16JT-125K
    ("ddram", 0,
        Subsignal("a", Pins("AF5 AF2 AD6 AC6 AD4 AB6 AE2 Y5 AA4 AE6 AE3 AD5 AB4 Y6"),
            IOStandard("SSTL15")),
        Subsignal("ba", Pins("AD3 AE1 AE5"), IOStandard("SSTL15")),
        Subsignal("ras_n", Pins("AC3"),      IOStandard("SSTL15")),
        Subsignal("cas_n", Pins("AC4"),      IOStandard("SSTL15")),
        Subsignal("we_n",  Pins("AF4"),      IOStandard("SSTL15")),
        #Subsignal("cs_n", Pins("--"),       IOStandard("SSTL15")),
        Subsignal("dm", Pins("V1 V3"),       IOStandard("SSTL15")),
        Subsignal("dq", Pins(
            "W1 V2 Y1 Y3 AC2 Y2 AB2 AA3",
            "U1 V4 U6 W3 V6 U2 U7 U5"),
            IOStandard("SSTL15")), # _T_DCI")),

        Subsignal("dqs_p", Pins("AB1 W6"), IOStandard("DIFF_SSTL15")), # _T_DCI")),
        Subsignal("dqs_n", Pins("AC1 W5"), IOStandard("DIFF_SSTL15")), # _T_DCI")),

        Subsignal("clk_p", Pins("AA5"),    IOStandard("DIFF_SSTL15")),
        Subsignal("clk_n", Pins("AB5"),    IOStandard("DIFF_SSTL15")),

        Subsignal("cke",   Pins("AD1"),    IOStandard("SSTL15")),
        Subsignal("odt",   Pins("AF3"),    IOStandard("SSTL15")),
        Subsignal("reset_n", Pins("W4"),   IOStandard("LVCMOS15")),
        Misc("SLEW=FAST"),
    ),
]

# The connectors are named after the daughterboard, not the core board
# because on the different core boards the names vary, but on the
# daughterboard they stay the same, which we need to connect the
# daughterboard peripherals to the core board.
# On this board J2 is U5 and J3 is U4
_connectors = [
    ("J2", {
         # odd row     even row
          7: "A8",    8: "A9",
          9: "B9",   10: "C9",
         11: "A10",  12: "B10",
         13: "D10",  14: "E10",
         15: "B11",  16: "B12",
         17: "C11",  18: "C12",
         19: "A12",  20: "A13",
         21: "D13",  22: "D14",
         23: "A14",  24: "B14",
         25: "C13",  26: "C14",
         27: "A15",  28: "B15",
         29: "D16",  30: "D15",
         31: "B16",  32: "C16",
         33: "A17",  34: "B17",
         35: "D18",  36: "E18",
         37: "C18",  38: "C17",
         39: "A19",  40: "A18",
         41: "B19",  42: "C19",
         43: "A20",  44: "B20",
         45: "D20",  46: "D19",
         47: "A24",  48: "A23",
         49: "E22",  50: "E21",
         51: "D24",  52: "D23",
         53: "D25",  54: "E25",
         55: "E26",  56: "F25",
         57: "B26",  58: "B25",
         59: "C26",  60: "D26",
    }),
    ("J3", {
        # odd row     even row
         7: "AD21",   8: "AE21",
         9: "AE22",  10: "AF22",
        11: "AE23",  12: "AF23",
        13: "V21",   14: "W21",
        15: "Y22",   16: "AA22",
        17: "AF24",  18: "AF25",
        19: "AB21",  20: "AC21",
        21: "AB22",  22: "AC22",
        23: "AD23",  24: "AD24",
        25: "AC23",  26: "AC24",
        27: "AD25",  28: "AE25",
        29: "AA23",  30: "AB24",
        31: "AA25",  32: "AB25",
        33: "Y23",   34: "AA24",
        35: "AD26",  36: "AE26",
        37: "AB26",  38: "AC26",
        39: "W23",   40: "W24",
        41: "Y25",   42: "Y26",
        43: "W25",   44: "W26",
        45: "U26",   46: "V26",
        47: "V23",   48: "V24",
        49: "U24",   50: "U25",
        51: "T22",   52: "T23",
        53: "R22",   54: "R23",
        55: "R25",   56: "P25",
        57: "P23",   58: "N23",
        59: "N26",   60: "M26",
    })
]

# Platform -----------------------------------------------------------------------------------------

class Platform(XilinxPlatform):
    default_clk_name   = "clk50"
    default_clk_period = 1e9/50e6

    core_resources_daughterboard = [
        ("onboard_led_1", 0, Pins("J26"), IOStandard("LVCMOS33")),
        ("onboard_led_2", 0, Pins("H26"), IOStandard("LVCMOS33")),
        ("cpu_reset", 0, Pins("AD21"), IOStandard("LVCMOS33")),
    ]

    core_resources_standalone = [
        ("user_led", 0, Pins("J26"), IOStandard("LVCMOS33")),
        ("user_led", 1, Pins("H26"), IOStandard("LVCMOS33")),
        ("cpu_reset", 0, Pins("AD21"), IOStandard("LVCMOS33")),
    ]

    def __init__(self, toolchain="vivado", with_daughterboard=False):
        device = "xc7k325tffg676-1"
        io = _io
        connectors = _connectors

        if with_daughterboard:
            io += self.core_resources_daughterboard
            from litex_boards.platforms.qmtech_daughterboard import QMTechDaughterboard
            daughterboard = QMTechDaughterboard(IOStandard("LVCMOS33"))
            io += daughterboard.io
            connectors += daughterboard.connectors
        else:
            io += self.core_resources_standalone

        XilinxPlatform.__init__(self, device, io, connectors, toolchain=toolchain)

        self.add_platform_command("set_property INTERNAL_VREF 0.750 [get_iobanks 34]")
        self.add_platform_command("set_property INTERNAL_VREF 0.90  [get_iobanks 33]")

    def create_programmer(self):
        bscan_spi = "bscan_spi_xc7k325t.bit"
        return OpenOCD("openocd_xc7_ft2232.cfg", bscan_spi)


    def do_finalize(self, fragment):
        XilinxPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk50", loose=True), 1e9/50e6)



#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2022 Yonggang Liu <ggang.liu@gmail.com>,
# SPDX-License-Identifier: BSD-2-Clause

from migen import *

from litex.gen import *

from litex_boards.platforms import alinx_ax7010

from litex.soc.interconnect import axi
from litex.soc.interconnect import wishbone

from litex.soc.cores.clock import *
from litex.soc.integration.soc_core import *
from litex.soc.integration.builder import *
from litex.soc.cores.led import LedChaser

# CRG ----------------------------------------------------------------------------------------------

class _CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq):
        self.rst    = Signal()
        self.cd_sys = ClockDomain()

        self.pll = pll = S7PLL(speedgrade=-1)
        self.comb += pll.reset.eq(self.rst)
        pll.register_clkin(platform.request("clk100"), 100e6)
        pll.create_clkout(self.cd_sys, sys_clk_freq)
        platform.add_false_path_constraints(self.cd_sys.clk, pll.clkin) # Ignore sys_clk to pll.clkin path created by SoC's rst.
        
# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    def __init__(self, sys_clk_freq=100e6, with_led_chaser=True, **kwargs):
        platform = alinx_ax7010.Platform()

        # CRG --------------------------------------------------------------------------------------
        self.crg = _CRG(platform, sys_clk_freq)

        # SoCCore ----------------------------------------------------------------------------------
        #if kwargs["uart_name"] == "serial": kwargs["uart_name"] = "usb_uart" # Use USB-UART Pmod on JB.
        kwargs["uart_name"] = "serial"
        SoCCore.__init__(self, platform, sys_clk_freq, ident="LiteX SoC on Alinx AX7010", **kwargs)
        
        # Leds -------------------------------------------------------------------------------------
        if with_led_chaser:
            self.leds = LedChaser(
                pads         = platform.request_all("user_led"),
                sys_clk_freq = sys_clk_freq)

# Build --------------------------------------------------------------------------------------------

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=alinx_ax7010.Platform, description="LiteX SoC on zynq xc7z010.")
    parser.add_target_argument("--sys-clk-freq", default=100e6, type=float, help="System clock frequency.")
    args = parser.parse_args()

    soc = BaseSoC(
        sys_clk_freq = args.sys_clk_freq,
        **parser.soc_argdict
    )
    builder = Builder(soc, **parser.builder_argdict)
    if args.build:
        builder.build(**parser.toolchain_argdict)

    if args.load:
        prog = soc.platform.create_programmer()
        prog.load_bitstream(builder.get_bitstream_filename(mode="sram"), device=1)

if __name__ == "__main__":
    main()



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2022 Raptor Engineering, LLC
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.lattice import LatticeECP5Platform
from litex.build.lattice.programmer import OpenOCDJTAGProgrammer

import os

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk / Rst (module)
    ("clk125",       0, Pins("B6"), IOStandard("LVCMOS33")),
    ("rst_n",        0, Pins("T3"), IOStandard("LVCMOS33")),

    # BMC serial (module)
    ("serial", 0,
        Subsignal("rx", Pins("A7"), IOStandard("LVCMOS33")),
        Subsignal("tx", Pins("B8"), IOStandard("LVCMOS33")),
    ),

    # Host serial (module)
    ("serial", 1,
        Subsignal("rx", Pins("C1"), IOStandard("LVCMOS33")),
        Subsignal("tx", Pins("C2"), IOStandard("LVCMOS33")),
        Subsignal("rts", Pins("C8"), IOStandard("LVCMOS33")),
        Subsignal("cts", Pins("D8"), IOStandard("LVCMOS33")),
    ),

    # DDR3 SDRAM (module)
    ("ddram", 0,
        Subsignal("a", Pins(
            "J1 K1 G2 H2 F1 G1 J4 J3 J5 K3 K2 H1 M5 K4 L4"),
            IOStandard("SSTL135_I")),
        Subsignal("ba",    Pins("K5 L5 M1"), IOStandard("SSTL135_I")),
        Subsignal("ras_n", Pins("L2"), IOStandard("SSTL135_I")),
        Subsignal("cas_n", Pins("N2"), IOStandard("SSTL135_I")),
        Subsignal("we_n",  Pins("N1"), IOStandard("SSTL135_I")),
        Subsignal("cs_n",  Pins("P5"), IOStandard("SSTL135_I")),
        Subsignal("dm", Pins("R20 N18 F20 E18"), IOStandard("SSTL135_I")),
        Subsignal("dq", Pins(
            "T20 U17 T18 U16 U19 T17 U20 U18",
            "L19 M18 L17 L16 L20 M19 L18 M20",
            "J20 K18 F19 K19 J19 J18 G20 K20",
            "G16 H18 H16 F18 J16 E17 J17 H17"),
            IOStandard("SSTL135_I"),
            Misc("TERMINATION=75")),
        Subsignal("dqs_p", Pins("T19 N16 G19 F17"), IOStandard("SSTL135D_I"),
            Misc("TERMINATION=OFF"),
            Misc("DIFFRESISTOR=100")),
        Subsignal("clk_p", Pins("P19 E16"), IOStandard("SSTL135D_I")),
        Subsignal("cke",   Pins("N5"), IOStandard("SSTL135_I")),
        Subsignal("odt",   Pins("M3"), IOStandard("SSTL135_I")),
        Subsignal("reset_n", Pins("L1"), IOStandard("SSTL135_I")),
        Misc("SLEWRATE=FAST"),
    ),

    # PCIe (module)
    ("pcie_x1", 0,
        Subsignal("clk_p", Pins("Y11")),
        Subsignal("clk_n", Pins("Y12")),
        Subsignal("rx_p",  Pins("Y5")),
        Subsignal("rx_n",  Pins("Y6")),
        Subsignal("tx_p",  Pins("W4")),
        Subsignal("tx_n",  Pins("W5")),
        Subsignal("perst", Pins("A6"), IOStandard("LVCMOS33")),
    ),

    # Inter-module SERDES (module)
    ("serdes_x2", 0,
        Subsignal("clk_p", Pins("Y19")),
        Subsignal("clk_n", Pins("W20")),
        Subsignal("rx_p",  Pins("Y14 Y16")),
        Subsignal("rx_n",  Pins("Y15 Y17")),
        Subsignal("tx_p",  Pins("W13 W17")),
        Subsignal("tx_n",  Pins("W14 W18")),
        Subsignal("perst", Pins("A6"), IOStandard("LVCMOS33")),
    ),

    # Bitstream Flash device (module)
    # Contains FPGA bistream, USRMCLK block required for clock output
    ("fpgaspiflash4x", 0,
        Subsignal("cs_n", Pins("R2")),
        Subsignal("dq",   Pins("W2 V2 Y2 W1")),
        IOStandard("LVCMOS33"),
        Misc("SLEWRATE=SLOW"),
        Misc("DRIVE=16"),
    ),

    # BMC firmware Flash device (carrier card)
    ("bmcspiflash4x", 0,
        Subsignal("cs_n", Pins("G5")),
        Subsignal("clk",  Pins("E5")),
        Subsignal("dq",   Pins("E3 F5 D2 H4")),
        IOStandard("LVCMOS33"),
        Misc("SLEWRATE=SLOW"),
        Misc("DRIVE=16"),
    ),

    # Host Flash device (carrier card)
    ("hostspiflash4x", 0,
        Subsignal("cs_n", Pins("E2")),
        Subsignal("clk",  Pins("G3")),
        Subsignal("dq",   Pins("F2 F3 D1 A2")),
        IOStandard("LVCMOS33"),
        Misc("SLEWRATE=SLOW"),
        Misc("DRIVE=16"),
    ),

    # I2C bus 1
    # 3-pin header (carrier card)
    ("i2c_master", 0,
        Subsignal("sda",       Pins("E4"), IOStandard("LVCMOS33"), Misc("PULLMODE=UP")),
        Subsignal("scl",       Pins("D5"), IOStandard("LVCMOS33"), Misc("PULLMODE=UP")),
    ),

    # I2C bus 2
    # 3-pin header (carrier card)
    ("i2c_master", 1,
        Subsignal("sda",       Pins("B1"), IOStandard("LVCMOS33"), Misc("PULLMODE=UP")),
        Subsignal("scl",       Pins("B2"), IOStandard("LVCMOS33"), Misc("PULLMODE=UP")),
    ),

    # I2C bus 3
    # 3-pin header (carrier card)
    ("i2c_master", 2,
        Subsignal("sda",       Pins("C7"), IOStandard("LVCMOS33"), Misc("PULLMODE=UP")),
        Subsignal("scl",       Pins("E8"), IOStandard("LVCMOS33"), Misc("PULLMODE=UP")),
    ),

    # I2C bus 4
    # GPIO expander 1 (module)
    ("i2c_master", 3,
        Subsignal("sda",       Pins("U1"), IOStandard("LVCMOS33"), Misc("PULLMODE=UP")),
        Subsignal("scl",       Pins("R1"), IOStandard("LVCMOS33"), Misc("PULLMODE=UP")),
    ),

    # I2C bus 5
    # GPIO expander 2 (module)
    ("i2c_master", 4,
        Subsignal("sda",       Pins("A12"), IOStandard("LVCMOS33"), Misc("PULLMODE=UP")),
        Subsignal("scl",       Pins("E12"), IOStandard("LVCMOS33"), Misc("PULLMODE=UP")),
    ),

    # I2C bus 9
    ("i2c_master", 5,
        Subsignal("sda",       Pins("R3"), IOStandard("LVCMOS33"), Misc("PULLMODE=UP")),
        Subsignal("scl",       Pins("U2"), IOStandard("LVCMOS33"), Misc("PULLMODE=UP")),
    ),

    # I2C bus 12
    # RTC + digital video + temperature sensor (module)
    # Clock generator + PMBus (carrier card)
    ("i2c_master", 6,
        Subsignal("sda",       Pins("V1"), IOStandard("LVCMOS33"), Misc("PULLMODE=UP")),
        Subsignal("scl",       Pins("T1"), IOStandard("LVCMOS33"), Misc("PULLMODE=UP")),
    ),

    # Host LPC interface (module)
    ("hostlpcslave", 0,
        Subsignal("frame_n",   Pins("D3"), Misc("PULLMODE=UP")),
        Subsignal("reset_n",   Pins("C3"), Misc("PULLMODE=UP")),
        Subsignal("addrdata",  Pins("C4 A3 B4 B3"), Misc("PULLMODE=UP")),
        Subsignal("serirq",    Pins("F4"), Misc("PULLMODE=UP")),
        Subsignal("clk",       Pins("H5"), Misc("PULLMODE=NONE")),
        IOStandard("LVCMOS33"),
        Misc("SLEWRATE=SLOW"),
        Misc("DRIVE=16"),
    ),

    # FSI (carrier card)
    ("openfsi_master", 0,
        Subsignal("clock",          Pins("A18"), IOStandard("LVCMOS33")),
        Subsignal("data",           Pins("B18"), IOStandard("LVCMOS33")),
        Subsignal("data_direction", Pins("T2"), IOStandard("LVCMOS33")),
    ),

    # RGMII Ethernet (module)
    ("eth_clocks", 0,
        Subsignal("tx", Pins("C11")),
        Subsignal("rx", Pins("A9")),
        IOStandard("LVCMOS33")
    ),
    ("eth", 0,
        # Reset is available on GPIO expander 2
        Subsignal("mdio",    Pins("D9")),
        Subsignal("mdc",     Pins("E6")),
        Subsignal("rx_ctl",  Pins("A8")),
        Subsignal("rx_data", Pins("E9 C9 D10 E10")),
        Subsignal("tx_ctl",  Pins("C10")),
        Subsignal("tx_data", Pins("B10 A10 B11 A11")),
        IOStandard("LVCMOS33")
    ),

    # Digital video (module)
    ("dvo", 0,
        Subsignal("r", Pins(
            "C14 E14 D14 E13 D13 C13 E11 C12")),
        Subsignal("g", Pins(
            "B19 B20 C17 C16 C15 D16 D15 E15")),
        Subsignal("b", Pins(
            "A14 A15 B15 A16 B16 A17 A19 B17")),
        Subsignal("de",      Pins("A13")),
        Subsignal("hsync_n", Pins("B13")),
        Subsignal("vsync_n", Pins("B12")),
        Subsignal("clk",     Pins("D11")),
        IOStandard("LVCMOS33")
    ),

    # 4-pin fan headers (carrier card)
    ("pwm_tach_pads", 0,
        Subsignal("pwm1", Pins("C5"), IOStandard("LVCMOS33")),
        Subsignal("pwm2", Pins("E1"), IOStandard("LVCMOS33")),
        Subsignal("pwm3", Pins("H3"), IOStandard("LVCMOS33")),
        Subsignal("pwm4", Pins("A5"), IOStandard("LVCMOS33")),
        Subsignal("tach1", Pins("C6"), IOStandard("LVCMOS33")),
        Subsignal("tach2", Pins("E7"), IOStandard("LVCMOS33")),
        Subsignal("tach3", Pins("D6"), IOStandard("LVCMOS33")),
        Subsignal("tach4", Pins("A4"), IOStandard("LVCMOS33")),
    ),
]

# Platform -----------------------------------------------------------------------------------------

class Platform(LatticeECP5Platform):
    default_clk_name   = "clk125"
    default_clk_period = 1e9/125e6

    def __init__(self, device="LFE5UM5G", speed_grade="6", toolchain="trellis", **kwargs):
        assert device in ["LFE5UM5G", "LFE5UM"]
        if device == "LFE5UM5G":
            speed_grade = "8"
        LatticeECP5Platform.__init__(self, device + "-85F-" + speed_grade + "CABGA381", _io, toolchain=toolchain, **kwargs)

    def request(self, *args, **kwargs):
        return LatticeECP5Platform.request(self, *args, **kwargs)

    def create_programmer(self):
        return OpenOCDJTAGProgrammer("openocd_evn_ecp5.cfg")

    def do_finalize(self, fragment):
        LatticeECP5Platform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk125", loose=True), 1e9/125e6)
        self.add_period_constraint(self.lookup_request("eth_clocks:rx", 0, loose=True), 1e9/125e6)
        self.add_period_constraint(self.lookup_request("eth_clocks:rx", 1, loose=True), 1e9/125e6)
